@@ -1,6 +1,6 @@
 /*
  * wpa_supplicant/hostapd / common helper functions, etc.
- * Copyright (c) 2002-2005, Jouni Malinen <jkmaline@cc.hut.fi>
+ * Copyright (c) 2002-2006, Jouni Malinen <jkmaline@cc.hut.fi>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 2 as
@@ -129,10 +129,10 @@ void wpa_get_ntp_timestamp(u8 *buf)
 
 	/* 64-bit NTP timestamp (time from 1900-01-01 00:00:00) */
 	os_get_time(&now);
-	sec = htonl(now.sec + 2208988800U); /* Epoch to 1900 */
+	sec = host_to_be32(now.sec + 2208988800U); /* Epoch to 1900 */
 	/* Estimate 2^32/10^6 = 4295 - 1/32 - 1/512 */
 	usec = now.usec;
-	usec = htonl(4295 * usec - (usec >> 5) - (usec >> 9));
+	usec = host_to_be32(4295 * usec - (usec >> 5) - (usec >> 9));
 	memcpy(buf, (u8 *) &sec, 4);
 	memcpy(buf + 4, (u8 *) &usec, 4);
 }
@@ -268,6 +268,45 @@ void wpa_hexdump_ascii_key(int level, const char *title, const u8 *buf,
 #endif /* CONFIG_NO_STDOUT_DEBUG */
 
 
+static inline int _wpa_snprintf_hex(char *buf, size_t buf_size, const u8 *data,
+				    size_t len, int uppercase)
+{
+	size_t i;
+	char *pos = buf, *end = buf + buf_size;
+	for (i = 0; i < len; i++) {
+		pos += snprintf(pos, end - pos, uppercase ? "%02X" : "%02x",
+				data[i]);
+	}
+	return pos - buf;
+}
+
+/**
+ * wpa_snprintf_hex - Print data as a hex string into a buffer
+ * @buf: Memory area to use as the output buffer
+ * @buf_size: Maximum buffer size in bytes (should be at least 2 * len + 1)
+ * @data: Data to be printed
+ * @len: Length of data in bytes
+ */
+int wpa_snprintf_hex(char *buf, size_t buf_size, const u8 *data, size_t len)
+{
+	return _wpa_snprintf_hex(buf, buf_size, data, len, 0);
+}
+
+
+/**
+ * wpa_snprintf_hex_uppercase - Print data as a upper case hex string into buf
+ * @buf: Memory area to use as the output buffer
+ * @buf_size: Maximum buffer size in bytes (should be at least 2 * len + 1)
+ * @data: Data to be printed
+ * @len: Length of data in bytes
+ */
+int wpa_snprintf_hex_uppercase(char *buf, size_t buf_size, const u8 *data,
+			       size_t len)
+{
+	return _wpa_snprintf_hex(buf, buf_size, data, len, 1);
+}
+
+
 #ifdef CONFIG_ANSI_C_EXTRA
 /*
  * Extremely simple (and likely inefficient) example implementation of some C
@@ -459,6 +498,15 @@ char * strdup(const char *s)
 }
 
 
+#ifdef _WIN32_WCE
+void perror(const char *s)
+{
+	wpa_printf(MSG_ERROR, "%s: GetLastError: %d",
+		   s, (int) GetLastError());
+}
+#endif /* _WIN32_WCE */
+
+
 int optind = 1;
 int optopt;
 char *optarg;
