@@ -1,6 +1,5 @@
 /*
- * Host AP (software wireless LAN access point) user space daemon for
- * Host AP kernel driver / Configuration file
+ * hostapd / Configuration file
  * Copyright (c) 2003-2005, Jouni Malinen <jkmaline@cc.hut.fi>
  *
  * This program is free software; you can redistribute it and/or modify
@@ -23,43 +22,60 @@
 #include "radius_client.h"
 
 
-static struct hostapd_config *hostapd_config_defaults(void)
+static void hostapd_config_defaults_bss(struct hostapd_bss_config *bss)
+{
+	memset(bss, 0, sizeof(*bss) + sizeof(struct hostapd_radius_servers));
+	bss->radius = (struct hostapd_radius_servers *) (bss + 1);
+
+	bss->logger_syslog_level = HOSTAPD_LEVEL_INFO;
+	bss->logger_stdout_level = HOSTAPD_LEVEL_INFO;
+	bss->logger_syslog = (unsigned int) -1;
+	bss->logger_stdout = (unsigned int) -1;
+
+	bss->auth_algs = HOSTAPD_AUTH_OPEN | HOSTAPD_AUTH_SHARED_KEY;
+
+	bss->wep_rekeying_period = 300;
+	bss->eap_reauth_period = 3600;
+
+	bss->wpa_group_rekey = 600;
+	bss->wpa_gmk_rekey = 86400;
+	bss->wpa_key_mgmt = WPA_KEY_MGMT_PSK;
+	bss->wpa_pairwise = WPA_CIPHER_TKIP;
+	bss->wpa_group = WPA_CIPHER_TKIP;
+
+	bss->radius_server_auth_port = 1812;
+	bss->ap_max_inactivity = AP_MAX_INACTIVITY;
+}
+
+
+static struct hostapd_config * hostapd_config_defaults(void)
 {
 	struct hostapd_config *conf;
+	struct hostapd_bss_config *bss;
 
-	conf = malloc(sizeof(*conf) + sizeof(struct hostapd_radius_servers));
-	if (conf == NULL) {
+	conf = malloc(sizeof(*conf));
+	bss = malloc(sizeof(*bss) + sizeof(struct hostapd_radius_servers));
+	if (conf == NULL || bss == NULL) {
 		printf("Failed to allocate memory for configuration data.\n");
+		free(conf);
+		free(bss);
 		return NULL;
 	}
-	memset(conf, 0, sizeof(*conf) + sizeof(struct hostapd_radius_servers));
-	conf->radius = (struct hostapd_radius_servers *) (conf + 1);
+	memset(conf, 0, sizeof(*conf));
 
 	/* set default driver based on configuration */
 	conf->driver = driver_lookup("default");
 	if (conf->driver == NULL) {
 		printf("No default driver registered!\n");
 		free(conf);
+		free(bss);
 		return NULL;
 	}
 
-	conf->wep_rekeying_period = 300;
-	conf->eap_reauth_period = 3600;
+	hostapd_config_defaults_bss(bss);
 
-	conf->logger_syslog_level = HOSTAPD_LEVEL_INFO;
-	conf->logger_stdout_level = HOSTAPD_LEVEL_INFO;
-	conf->logger_syslog = (unsigned int) -1;
-	conf->logger_stdout = (unsigned int) -1;
-
-	conf->auth_algs = HOSTAPD_AUTH_OPEN | HOSTAPD_AUTH_SHARED_KEY;
-
-	conf->wpa_group_rekey = 600;
-	conf->wpa_gmk_rekey = 86400;
-	conf->wpa_key_mgmt = WPA_KEY_MGMT_PSK;
-	conf->wpa_pairwise = WPA_CIPHER_TKIP;
-	conf->wpa_group = WPA_CIPHER_TKIP;
-
-	conf->radius_server_auth_port = 1812;
+	conf->num_bss = 1;
+	conf->bss = bss;
 
 	return conf;
 }
@@ -151,7 +167,7 @@ static int hostapd_config_read_maclist(const char *fname, macaddr **acl,
 
 
 static int hostapd_config_read_wpa_psk(const char *fname,
-				       struct hostapd_config *conf)
+				       struct hostapd_ssid *ssid)
 {
 	FILE *f;
 	char buf[128], *pos;
@@ -217,7 +233,7 @@ static int hostapd_config_read_wpa_psk(const char *fname,
 		if (len == 64 && hexstr2bin(pos, psk->psk, PMK_LEN) == 0)
 			ok = 1;
 		else if (len >= 8 && len < 64) {
-			pbkdf2_sha1(pos, conf->ssid, conf->ssid_len,
+			pbkdf2_sha1(pos, ssid->ssid, ssid->ssid_len,
 				    4096, psk->psk, PMK_LEN);
 			ok = 1;
 		}
@@ -229,8 +245,8 @@ static int hostapd_config_read_wpa_psk(const char *fname,
 			break;
 		}
 
-		psk->next = conf->wpa_psk;
-		conf->wpa_psk = psk;
+		psk->next = ssid->wpa_psk;
+		ssid->wpa_psk = psk;
 	}
 
 	fclose(f);
@@ -239,42 +255,46 @@ static int hostapd_config_read_wpa_psk(const char *fname,
 }
 
 
-int hostapd_setup_wpa_psk(struct hostapd_config *conf)
+int hostapd_setup_wpa_psk(struct hostapd_bss_config *conf)
 {
-	if (conf->wpa_passphrase != NULL) {
-		if (conf->wpa_psk != NULL) {
+	struct hostapd_ssid *ssid = &conf->ssid;
+
+	if (ssid->wpa_passphrase != NULL) {
+		if (ssid->wpa_psk != NULL) {
 			printf("Warning: both WPA PSK and passphrase set. "
 			       "Using passphrase.\n");
-			free(conf->wpa_psk);
+			free(ssid->wpa_psk);
 		}
-		conf->wpa_psk = malloc(sizeof(struct hostapd_wpa_psk));
-		if (conf->wpa_psk == NULL) {
+		ssid->wpa_psk = malloc(sizeof(struct hostapd_wpa_psk));
+		if (ssid->wpa_psk == NULL) {
 			printf("Unable to alloc space for PSK\n");
 			return -1;
 		}
 		wpa_hexdump_ascii(MSG_DEBUG, "SSID",
-				  (u8 *) conf->ssid, conf->ssid_len);
+				  (u8 *) ssid->ssid, ssid->ssid_len);
 		wpa_hexdump_ascii(MSG_DEBUG, "PSK (ASCII passphrase)",
-				  (u8 *) conf->wpa_passphrase,
-				  strlen(conf->wpa_passphrase));
-		memset(conf->wpa_psk, 0, sizeof(struct hostapd_wpa_psk));
-		pbkdf2_sha1(conf->wpa_passphrase,
-			    conf->ssid, conf->ssid_len,
-			    4096, conf->wpa_psk->psk, PMK_LEN);
+				  (u8 *) ssid->wpa_passphrase,
+				  strlen(ssid->wpa_passphrase));
+		memset(ssid->wpa_psk, 0, sizeof(struct hostapd_wpa_psk));
+		pbkdf2_sha1(ssid->wpa_passphrase,
+			    ssid->ssid, ssid->ssid_len,
+			    4096, ssid->wpa_psk->psk, PMK_LEN);
 		wpa_hexdump(MSG_DEBUG, "PSK (from passphrase)",
-			    conf->wpa_psk->psk, PMK_LEN);
-		conf->wpa_psk->group = 1;
+			    ssid->wpa_psk->psk, PMK_LEN);
+		ssid->wpa_psk->group = 1;
 
-		memset(conf->wpa_passphrase, 0, strlen(conf->wpa_passphrase));
-		free(conf->wpa_passphrase);
-		conf->wpa_passphrase = 0;
+		memset(ssid->wpa_passphrase, 0,
+		       strlen(ssid->wpa_passphrase));
+		free(ssid->wpa_passphrase);
+		ssid->wpa_passphrase = 0;
 	}
 
-	if (conf->wpa_psk_file) {
-		if (hostapd_config_read_wpa_psk(conf->wpa_psk_file, conf))
+	if (ssid->wpa_psk_file) {
+		if (hostapd_config_read_wpa_psk(ssid->wpa_psk_file,
+						&conf->ssid))
 			return -1;
-		free(conf->wpa_psk_file);
-		conf->wpa_psk_file = NULL;
+		free(ssid->wpa_psk_file);
+		ssid->wpa_psk_file = NULL;
 	}
 
 	return 0;
@@ -283,7 +303,7 @@ int hostapd_setup_wpa_psk(struct hostapd_config *conf)
 
 #ifdef EAP_SERVER
 static int hostapd_config_read_eap_user(const char *fname,
-					struct hostapd_config *conf)
+					struct hostapd_bss_config *conf)
 {
 	FILE *f;
 	char buf[512], *pos, *start, *pos2;
@@ -384,8 +404,13 @@ static int hostapd_config_read_eap_user(const char *fname,
 			if (pos2) {
 				*pos2++ = '\0';
 			}
-			user->methods[num_methods] = eap_get_type(start);
-			if (user->methods[num_methods] == EAP_TYPE_NONE) {
+			user->methods[num_methods].method =
+				eap_get_type(start, &user->methods[num_methods]
+					     .vendor);
+			if (user->methods[num_methods].vendor ==
+			    EAP_VENDOR_IETF &&
+			    user->methods[num_methods].method == EAP_TYPE_NONE)
+			{
 				printf("Unsupported EAP type '%s' on line %d "
 				       "in '%s'\n", start, line, fname);
 				goto failed;
@@ -653,7 +678,7 @@ static int hostapd_config_parse_cipher(int line, const char *value)
 }
 
 
-static int hostapd_config_check(struct hostapd_config *conf)
+static int hostapd_config_check(struct hostapd_bss_config *conf)
 {
 	if (conf->ieee802_1x && !conf->eap_server &&
 	    !conf->radius->auth_servers) {
@@ -663,8 +688,8 @@ static int hostapd_config_check(struct hostapd_config *conf)
 	}
 
 	if (conf->wpa && (conf->wpa_key_mgmt & WPA_KEY_MGMT_PSK) &&
-	    conf->wpa_psk == NULL && conf->wpa_passphrase == NULL &&
-	    conf->wpa_psk_file == NULL) {
+	    conf->ssid.wpa_psk == NULL && conf->ssid.wpa_passphrase == NULL &&
+	    conf->ssid.wpa_psk_file == NULL) {
 		printf("WPA-PSK enabled, but PSK or passphrase is not "
 		       "configured.\n");
 		return -1;
@@ -674,17 +699,49 @@ static int hostapd_config_check(struct hostapd_config *conf)
 }
 
 
+static int hostapd_config_read_wep(struct hostapd_wep_keys *wep, int keyidx,
+				   char *val)
+{
+	size_t len = strlen(val);
+
+	if (keyidx < 0 || keyidx > 3 || wep->key[keyidx] != NULL)
+		return -1;
+
+	if (val[0] == '"') {
+		if (len < 2 || val[len - 1] != '"')
+			return -1;
+		len -= 2;
+		wep->key[keyidx] = malloc(len);
+		if (wep->key[keyidx] == NULL)
+			return -1;
+		memcpy(wep->key[keyidx], val + 1, len);
+		wep->len[keyidx] = len;
+	} else {
+		if (len & 1)
+			return -1;
+		len /= 2;
+		wep->key[keyidx] = malloc(len);
+		if (wep->key[keyidx] == NULL)
+			return -1;
+		wep->len[keyidx] = len;
+		if (hexstr2bin(val, wep->key[keyidx], len) < 0)
+			return -1;
+	}
+
+	wep->keys_set++;
+
+	return 0;
+}
+
+
 struct hostapd_config * hostapd_config_read(const char *fname)
 {
 	struct hostapd_config *conf;
+	struct hostapd_bss_config *bss;
 	FILE *f;
 	char buf[256], *pos;
-	int line = 0;
+	int line = 0, i;
 	int errors = 0;
-	char *accept_mac_file = NULL, *deny_mac_file = NULL;
-#ifdef EAP_SERVER
-	char *eap_user_file = NULL;
-#endif /* EAP_SERVER */
 
 	f = fopen(fname, "r");
 	if (f == NULL) {
@@ -698,8 +755,10 @@ struct hostapd_config * hostapd_config_read(const char *fname)
 		fclose(f);
 		return NULL;
 	}
+	bss = conf->last_bss = conf->bss;
 
 	while (fgets(buf, sizeof(buf), f)) {
+		bss = conf->last_bss;
 		line++;
 
 		if (buf[0] == '#')
@@ -725,10 +784,10 @@ struct hostapd_config * hostapd_config_read(const char *fname)
 		pos++;
 
 		if (strcmp(buf, "interface") == 0) {
-			snprintf(conf->iface, sizeof(conf->iface), "%s", pos);
+			snprintf(conf->bss[0].iface,
+				 sizeof(conf->bss[0].iface), "%s", pos);
 		} else if (strcmp(buf, "bridge") == 0) {
-			snprintf(conf->bridge, sizeof(conf->bridge), "%s",
-				 pos);
+			snprintf(bss->bridge, sizeof(bss->bridge), "%s", pos);
 		} else if (strcmp(buf, "driver") == 0) {
 			conf->driver = driver_lookup(pos);
 			if (conf->driver == NULL) {
@@ -737,174 +796,178 @@ struct hostapd_config * hostapd_config_read(const char *fname)
 				errors++;
 			}
 		} else if (strcmp(buf, "debug") == 0) {
-			conf->debug = atoi(pos);
+			bss->debug = atoi(pos);
 		} else if (strcmp(buf, "logger_syslog_level") == 0) {
-			conf->logger_syslog_level = atoi(pos);
+			bss->logger_syslog_level = atoi(pos);
 		} else if (strcmp(buf, "logger_stdout_level") == 0) {
-			conf->logger_stdout_level = atoi(pos);
+			bss->logger_stdout_level = atoi(pos);
 		} else if (strcmp(buf, "logger_syslog") == 0) {
-			conf->logger_syslog = atoi(pos);
+			bss->logger_syslog = atoi(pos);
 		} else if (strcmp(buf, "logger_stdout") == 0) {
-			conf->logger_stdout = atoi(pos);
+			bss->logger_stdout = atoi(pos);
 		} else if (strcmp(buf, "dump_file") == 0) {
-			conf->dump_log_name = strdup(pos);
+			bss->dump_log_name = strdup(pos);
 		} else if (strcmp(buf, "ssid") == 0) {
-			conf->ssid_len = strlen(pos);
-			if (conf->ssid_len >= HOSTAPD_SSID_LEN ||
-			    conf->ssid_len < 1) {
+			bss->ssid.ssid_len = strlen(pos);
+			if (bss->ssid.ssid_len >= HOSTAPD_MAX_SSID_LEN ||
+			    bss->ssid.ssid_len < 1) {
 				printf("Line %d: invalid SSID '%s'\n", line,
 				       pos);
 				errors++;
 			}
-			memcpy(conf->ssid, pos, conf->ssid_len);
-			conf->ssid[conf->ssid_len] = '\0';
-			conf->ssid_set = 1;
+			memcpy(bss->ssid.ssid, pos, bss->ssid.ssid_len);
+			bss->ssid.ssid[bss->ssid.ssid_len] = '\0';
+			bss->ssid.ssid_set = 1;
 		} else if (strcmp(buf, "macaddr_acl") == 0) {
-			conf->macaddr_acl = atoi(pos);
-			if (conf->macaddr_acl != ACCEPT_UNLESS_DENIED &&
-			    conf->macaddr_acl != DENY_UNLESS_ACCEPTED &&
-			    conf->macaddr_acl != USE_EXTERNAL_RADIUS_AUTH) {
+			bss->macaddr_acl = atoi(pos);
+			if (bss->macaddr_acl != ACCEPT_UNLESS_DENIED &&
+			    bss->macaddr_acl != DENY_UNLESS_ACCEPTED &&
+			    bss->macaddr_acl != USE_EXTERNAL_RADIUS_AUTH) {
 				printf("Line %d: unknown macaddr_acl %d\n",
-				       line, conf->macaddr_acl);
+				       line, bss->macaddr_acl);
 			}
 		} else if (strcmp(buf, "accept_mac_file") == 0) {
-			accept_mac_file = strdup(pos);
-			if (!accept_mac_file) {
-				printf("Line %d: allocation failed\n", line);
+			if (hostapd_config_read_maclist(pos, &bss->accept_mac,
+							&bss->num_accept_mac))
+			{
+				printf("Line %d: Failed to read "
+				       "accept_mac_file '%s'\n",
+				       line, pos);
 				errors++;
 			}
 		} else if (strcmp(buf, "deny_mac_file") == 0) {
-			deny_mac_file = strdup(pos);
-			if (!deny_mac_file) {
-				printf("Line %d: allocation failed\n", line);
+			if (hostapd_config_read_maclist(pos, &bss->deny_mac,
+							&bss->num_deny_mac))
+			{
+				printf("Line %d: Failed to read "
+				       "deny_mac_file '%s'\n",
+				       line, pos);
 				errors++;
 			}
+		} else if (strcmp(buf, "ap_max_inactivity") == 0) {
+			bss->ap_max_inactivity = atoi(pos);
 		} else if (strcmp(buf, "assoc_ap_addr") == 0) {
-			if (hwaddr_aton(pos, conf->assoc_ap_addr)) {
+			if (hwaddr_aton(pos, bss->assoc_ap_addr)) {
 				printf("Line %d: invalid MAC address '%s'\n",
 				       line, pos);
 				errors++;
 			}
-			conf->assoc_ap = 1;
+			bss->assoc_ap = 1;
 		} else if (strcmp(buf, "ieee8021x") == 0) {
-			conf->ieee802_1x = atoi(pos);
+			bss->ieee802_1x = atoi(pos);
 #ifdef EAP_SERVER
 		} else if (strcmp(buf, "eap_authenticator") == 0) {
-			conf->eap_server = atoi(pos);
+			bss->eap_server = atoi(pos);
 			printf("Line %d: obsolete eap_authenticator used; "
 			       "this has been renamed to eap_server\n", line);
 		} else if (strcmp(buf, "eap_server") == 0) {
-			conf->eap_server = atoi(pos);
+			bss->eap_server = atoi(pos);
 		} else if (strcmp(buf, "eap_user_file") == 0) {
-			free(eap_user_file);
-			eap_user_file = strdup(pos);
-			if (!eap_user_file) {
-				printf("Line %d: allocation failed\n", line);
+			if (hostapd_config_read_eap_user(pos, bss))
 				errors++;
-			}
 		} else if (strcmp(buf, "ca_cert") == 0) {
-			free(conf->ca_cert);
-			conf->ca_cert = strdup(pos);
+			free(bss->ca_cert);
+			bss->ca_cert = strdup(pos);
 		} else if (strcmp(buf, "server_cert") == 0) {
-			free(conf->server_cert);
-			conf->server_cert = strdup(pos);
+			free(bss->server_cert);
+			bss->server_cert = strdup(pos);
 		} else if (strcmp(buf, "private_key") == 0) {
-			free(conf->private_key);
-			conf->private_key = strdup(pos);
+			free(bss->private_key);
+			bss->private_key = strdup(pos);
 		} else if (strcmp(buf, "private_key_passwd") == 0) {
-			free(conf->private_key_passwd);
-			conf->private_key_passwd = strdup(pos);
+			free(bss->private_key_passwd);
+			bss->private_key_passwd = strdup(pos);
 		} else if (strcmp(buf, "check_crl") == 0) {
-			conf->check_crl = atoi(pos);
+			bss->check_crl = atoi(pos);
 #ifdef EAP_SIM
 		} else if (strcmp(buf, "eap_sim_db") == 0) {
-			free(conf->eap_sim_db);
-			conf->eap_sim_db = strdup(pos);
+			free(bss->eap_sim_db);
+			bss->eap_sim_db = strdup(pos);
 #endif /* EAP_SIM */
 #endif /* EAP_SERVER */
 		} else if (strcmp(buf, "eap_message") == 0) {
 			char *term;
-			conf->eap_req_id_text = strdup(pos);
-			if (conf->eap_req_id_text == NULL) {
+			bss->eap_req_id_text = strdup(pos);
+			if (bss->eap_req_id_text == NULL) {
 				printf("Line %d: Failed to allocate memory "
 				       "for eap_req_id_text\n", line);
 				errors++;
 				continue;
 			}
-			conf->eap_req_id_text_len =
-				strlen(conf->eap_req_id_text);
-			term = strstr(conf->eap_req_id_text, "\\0");
+			bss->eap_req_id_text_len =
+				strlen(bss->eap_req_id_text);
+			term = strstr(bss->eap_req_id_text, "\\0");
 			if (term) {
 				*term++ = '\0';
 				memmove(term, term + 1,
-					conf->eap_req_id_text_len -
-					(term - conf->eap_req_id_text) - 1);
-				conf->eap_req_id_text_len--;
+					bss->eap_req_id_text_len -
+					(term - bss->eap_req_id_text) - 1);
+				bss->eap_req_id_text_len--;
 			}
 		} else if (strcmp(buf, "wep_key_len_broadcast") == 0) {
-			conf->default_wep_key_len = atoi(pos);
-			if (conf->default_wep_key_len > 13) {
+			bss->default_wep_key_len = atoi(pos);
+			if (bss->default_wep_key_len > 13) {
 				printf("Line %d: invalid WEP key len %lu "
 				       "(= %lu bits)\n", line,
 				       (unsigned long)
-				       conf->default_wep_key_len,
+				       bss->default_wep_key_len,
 				       (unsigned long)
-				       conf->default_wep_key_len * 8);
+				       bss->default_wep_key_len * 8);
 				errors++;
 			}
 		} else if (strcmp(buf, "wep_key_len_unicast") == 0) {
-			conf->individual_wep_key_len = atoi(pos);
-			if (conf->individual_wep_key_len < 0 ||
-			    conf->individual_wep_key_len > 13) {
+			bss->individual_wep_key_len = atoi(pos);
+			if (bss->individual_wep_key_len < 0 ||
+			    bss->individual_wep_key_len > 13) {
 				printf("Line %d: invalid WEP key len %d "
 				       "(= %d bits)\n", line,
-				       conf->individual_wep_key_len,
-				       conf->individual_wep_key_len * 8);
+				       bss->individual_wep_key_len,
+				       bss->individual_wep_key_len * 8);
 				errors++;
 			}
 		} else if (strcmp(buf, "wep_rekey_period") == 0) {
-			conf->wep_rekeying_period = atoi(pos);
-			if (conf->wep_rekeying_period < 0) {
+			bss->wep_rekeying_period = atoi(pos);
+			if (bss->wep_rekeying_period < 0) {
 				printf("Line %d: invalid period %d\n",
-				       line, conf->wep_rekeying_period);
+				       line, bss->wep_rekeying_period);
 				errors++;
 			}
 		} else if (strcmp(buf, "eap_reauth_period") == 0) {
-			conf->eap_reauth_period = atoi(pos);
-			if (conf->eap_reauth_period < 0) {
+			bss->eap_reauth_period = atoi(pos);
+			if (bss->eap_reauth_period < 0) {
 				printf("Line %d: invalid period %d\n",
-				       line, conf->eap_reauth_period);
+				       line, bss->eap_reauth_period);
 				errors++;
 			}
 		} else if (strcmp(buf, "eapol_key_index_workaround") == 0) {
-			conf->eapol_key_index_workaround = atoi(pos);
+			bss->eapol_key_index_workaround = atoi(pos);
 #ifdef CONFIG_IAPP
 		} else if (strcmp(buf, "iapp_interface") == 0) {
-			conf->ieee802_11f = 1;
-			snprintf(conf->iapp_iface, sizeof(conf->iapp_iface),
+			bss->ieee802_11f = 1;
+			snprintf(bss->iapp_iface, sizeof(bss->iapp_iface),
 				 "%s", pos);
 #endif /* CONFIG_IAPP */
 		} else if (strcmp(buf, "own_ip_addr") == 0) {
-			if (hostapd_parse_ip_addr(pos, &conf->own_ip_addr)) {
+			if (hostapd_parse_ip_addr(pos, &bss->own_ip_addr)) {
 				printf("Line %d: invalid IP address '%s'\n",
 				       line, pos);
 				errors++;
 			}
 		} else if (strcmp(buf, "nas_identifier") == 0) {
-			conf->nas_identifier = strdup(pos);
+			bss->nas_identifier = strdup(pos);
 		} else if (strcmp(buf, "auth_server_addr") == 0) {
 			if (hostapd_config_read_radius_addr(
-				    &conf->radius->auth_servers,
-				    &conf->radius->num_auth_servers, pos, 1812,
-				    &conf->radius->auth_server)) {
+				    &bss->radius->auth_servers,
+				    &bss->radius->num_auth_servers, pos, 1812,
+				    &bss->radius->auth_server)) {
 				printf("Line %d: invalid IP address '%s'\n",
 				       line, pos);
 				errors++;
 			}
-		} else if (conf->radius->auth_server &&
+		} else if (bss->radius->auth_server &&
 			   strcmp(buf, "auth_server_port") == 0) {
-			conf->radius->auth_server->port = atoi(pos);
-		} else if (conf->radius->auth_server &&
+			bss->radius->auth_server->port = atoi(pos);
+		} else if (bss->radius->auth_server &&
 			   strcmp(buf, "auth_server_shared_secret") == 0) {
 			int len = strlen(pos);
 			if (len == 0) {
@@ -913,22 +976,22 @@ struct hostapd_config * hostapd_config_read(const char *fname)
 				       "allowed.\n", line);
 				errors++;
 			}
-			conf->radius->auth_server->shared_secret =
+			bss->radius->auth_server->shared_secret =
 				(u8 *) strdup(pos);
-			conf->radius->auth_server->shared_secret_len = len;
+			bss->radius->auth_server->shared_secret_len = len;
 		} else if (strcmp(buf, "acct_server_addr") == 0) {
 			if (hostapd_config_read_radius_addr(
-				    &conf->radius->acct_servers,
-				    &conf->radius->num_acct_servers, pos, 1813,
-				    &conf->radius->acct_server)) {
+				    &bss->radius->acct_servers,
+				    &bss->radius->num_acct_servers, pos, 1813,
+				    &bss->radius->acct_server)) {
 				printf("Line %d: invalid IP address '%s'\n",
 				       line, pos);
 				errors++;
 			}
-		} else if (conf->radius->acct_server &&
+		} else if (bss->radius->acct_server &&
 			   strcmp(buf, "acct_server_port") == 0) {
-			conf->radius->acct_server->port = atoi(pos);
-		} else if (conf->radius->acct_server &&
+			bss->radius->acct_server->port = atoi(pos);
+		} else if (bss->radius->acct_server &&
 			   strcmp(buf, "acct_server_shared_secret") == 0) {
 			int len = strlen(pos);
 			if (len == 0) {
@@ -937,29 +1000,29 @@ struct hostapd_config * hostapd_config_read(const char *fname)
 				       "allowed.\n", line);
 				errors++;
 			}
-			conf->radius->acct_server->shared_secret =
+			bss->radius->acct_server->shared_secret =
 				(u8 *) strdup(pos);
-			conf->radius->acct_server->shared_secret_len = len;
+			bss->radius->acct_server->shared_secret_len = len;
 		} else if (strcmp(buf, "radius_retry_primary_interval") == 0) {
-			conf->radius->retry_primary_interval = atoi(pos);
+			bss->radius->retry_primary_interval = atoi(pos);
 		} else if (strcmp(buf, "radius_acct_interim_interval") == 0) {
-			conf->radius->acct_interim_interval = atoi(pos);
+			bss->radius->acct_interim_interval = atoi(pos);
 		} else if (strcmp(buf, "auth_algs") == 0) {
-			conf->auth_algs = atoi(pos);
-			if (conf->auth_algs == 0) {
+			bss->auth_algs = atoi(pos);
+			if (bss->auth_algs == 0) {
 				printf("Line %d: no authentication algorithms "
 				       "allowed\n",
 				       line);
 				errors++;
 			}
 		} else if (strcmp(buf, "wpa") == 0) {
-			conf->wpa = atoi(pos);
+			bss->wpa = atoi(pos);
 		} else if (strcmp(buf, "wpa_group_rekey") == 0) {
-			conf->wpa_group_rekey = atoi(pos);
+			bss->wpa_group_rekey = atoi(pos);
 		} else if (strcmp(buf, "wpa_strict_rekey") == 0) {
-			conf->wpa_strict_rekey = atoi(pos);
+			bss->wpa_strict_rekey = atoi(pos);
 		} else if (strcmp(buf, "wpa_gmk_rekey") == 0) {
-			conf->wpa_gmk_rekey = atoi(pos);
+			bss->wpa_gmk_rekey = atoi(pos);
 		} else if (strcmp(buf, "wpa_passphrase") == 0) {
 			int len = strlen(pos);
 			if (len < 8 || len > 63) {
@@ -967,70 +1030,72 @@ struct hostapd_config * hostapd_config_read(const char *fname)
 				       " %d (expected 8..63)\n", line, len);
 				errors++;
 			} else {
-				free(conf->wpa_passphrase);
-				conf->wpa_passphrase = strdup(pos);
+				free(bss->ssid.wpa_passphrase);
+				bss->ssid.wpa_passphrase = strdup(pos);
 			}
 		} else if (strcmp(buf, "wpa_psk") == 0) {
-			free(conf->wpa_psk);
-			conf->wpa_psk = malloc(sizeof(struct hostapd_wpa_psk));
-			if (conf->wpa_psk) {
-				memset(conf->wpa_psk, 0,
+			free(bss->ssid.wpa_psk);
+			bss->ssid.wpa_psk =
+				malloc(sizeof(struct hostapd_wpa_psk));
+			if (bss->ssid.wpa_psk) {
+				memset(bss->ssid.wpa_psk, 0,
 				       sizeof(struct hostapd_wpa_psk));
 			}
-			if (conf->wpa_psk == NULL)
+			if (bss->ssid.wpa_psk == NULL)
 				errors++;
-			else if (hexstr2bin(pos, conf->wpa_psk->psk, PMK_LEN)
-				 || pos[PMK_LEN * 2] != '\0') {
+			else if (hexstr2bin(pos, bss->ssid.wpa_psk->psk,
+					    PMK_LEN) ||
+				 pos[PMK_LEN * 2] != '\0') {
 				printf("Line %d: Invalid PSK '%s'.\n", line,
 				       pos);
 				errors++;
 			} else {
-				conf->wpa_psk->group = 1;
+				bss->ssid.wpa_psk->group = 1;
 			}
 		} else if (strcmp(buf, "wpa_psk_file") == 0) {
-			free(conf->wpa_psk_file);
-			conf->wpa_psk_file = strdup(pos);
-			if (!conf->wpa_psk_file) {
+			free(bss->ssid.wpa_psk_file);
+			bss->ssid.wpa_psk_file = strdup(pos);
+			if (!bss->ssid.wpa_psk_file) {
 				printf("Line %d: allocation failed\n", line);
 				errors++;
 			}
 		} else if (strcmp(buf, "wpa_key_mgmt") == 0) {
-			conf->wpa_key_mgmt =
+			bss->wpa_key_mgmt =
 				hostapd_config_parse_key_mgmt(line, pos);
-			if (conf->wpa_key_mgmt == -1)
+			if (bss->wpa_key_mgmt == -1)
 				errors++;
 		} else if (strcmp(buf, "wpa_pairwise") == 0) {
-			conf->wpa_pairwise =
+			bss->wpa_pairwise =
 				hostapd_config_parse_cipher(line, pos);
-			if (conf->wpa_pairwise == -1 ||
-			    conf->wpa_pairwise == 0)
+			if (bss->wpa_pairwise == -1 ||
+			    bss->wpa_pairwise == 0)
 				errors++;
-			else if (conf->wpa_pairwise &
+			else if (bss->wpa_pairwise &
 				 (WPA_CIPHER_NONE | WPA_CIPHER_WEP40 |
 				  WPA_CIPHER_WEP104)) {
 				printf("Line %d: unsupported pairwise "
 				       "cipher suite '%s'\n",
-				       conf->wpa_pairwise, pos);
+				       bss->wpa_pairwise, pos);
 				errors++;
 			} else {
-				if (conf->wpa_pairwise & WPA_CIPHER_TKIP)
-					conf->wpa_group = WPA_CIPHER_TKIP;
+				if (bss->wpa_pairwise & WPA_CIPHER_TKIP)
+					bss->wpa_group = WPA_CIPHER_TKIP;
 				else
-					conf->wpa_group = WPA_CIPHER_CCMP;
+					bss->wpa_group = WPA_CIPHER_CCMP;
 			}
 #ifdef CONFIG_RSN_PREAUTH
 		} else if (strcmp(buf, "rsn_preauth") == 0) {
-			conf->rsn_preauth = atoi(pos);
+			bss->rsn_preauth = atoi(pos);
 		} else if (strcmp(buf, "rsn_preauth_interfaces") == 0) {
-			conf->rsn_preauth_interfaces = strdup(pos);
+			bss->rsn_preauth_interfaces = strdup(pos);
 #endif /* CONFIG_RSN_PREAUTH */
 #ifdef CONFIG_STAKEY
 		} else if (strcmp(buf, "stakey") == 0) {
-			conf->stakey = atoi(pos);
+			bss->stakey = atoi(pos);
 #endif /* CONFIG_STAKEY */
 		} else if (strcmp(buf, "ctrl_interface") == 0) {
-			free(conf->ctrl_interface);
-			conf->ctrl_interface = strdup(pos);
+			free(bss->ctrl_interface);
+			bss->ctrl_interface = strdup(pos);
 		} else if (strcmp(buf, "ctrl_interface_group") == 0) {
 			struct group *grp;
 			char *endp;
@@ -1038,39 +1103,58 @@ struct hostapd_config * hostapd_config_read(const char *fname)
 
 			grp = getgrnam(group);
 			if (grp) {
-				conf->ctrl_interface_gid = grp->gr_gid;
-				conf->ctrl_interface_gid_set = 1;
+				bss->ctrl_interface_gid = grp->gr_gid;
+				bss->ctrl_interface_gid_set = 1;
 				wpa_printf(MSG_DEBUG, "ctrl_interface_group=%d"
 					   " (from group name '%s')",
-					   conf->ctrl_interface_gid, group);
+					   bss->ctrl_interface_gid, group);
 				continue;
 			}
 
 			/* Group name not found - try to parse this as gid */
-			conf->ctrl_interface_gid = strtol(group, &endp, 10);
+			bss->ctrl_interface_gid = strtol(group, &endp, 10);
 			if (*group == '\0' || *endp != '\0') {
 				wpa_printf(MSG_DEBUG, "Line %d: Invalid group "
 					   "'%s'", line, group);
 				errors++;
 				continue;
 			}
-			conf->ctrl_interface_gid_set = 1;
+			bss->ctrl_interface_gid_set = 1;
 			wpa_printf(MSG_DEBUG, "ctrl_interface_group=%d",
-				   conf->ctrl_interface_gid);
+				   bss->ctrl_interface_gid);
 #ifdef RADIUS_SERVER
 		} else if (strcmp(buf, "radius_server_clients") == 0) {
-			free(conf->radius_server_clients);
-			conf->radius_server_clients = strdup(pos);
+			free(bss->radius_server_clients);
+			bss->radius_server_clients = strdup(pos);
 		} else if (strcmp(buf, "radius_server_auth_port") == 0) {
-			conf->radius_server_auth_port = atoi(pos);
+			bss->radius_server_auth_port = atoi(pos);
 		} else if (strcmp(buf, "radius_server_ipv6") == 0) {
-			conf->radius_server_ipv6 = atoi(pos);
+			bss->radius_server_ipv6 = atoi(pos);
 #endif /* RADIUS_SERVER */
 		} else if (strcmp(buf, "test_socket") == 0) {
-			free(conf->test_socket);
-			conf->test_socket = strdup(pos);
+			free(bss->test_socket);
+			bss->test_socket = strdup(pos);
 		} else if (strcmp(buf, "use_pae_group_addr") == 0) {
-			conf->use_pae_group_addr = atoi(pos);
+			bss->use_pae_group_addr = atoi(pos);
+		} else if (strcmp(buf, "wep_default_key") == 0) {
+			bss->ssid.wep.idx = atoi(pos);
+			if (bss->ssid.wep.idx > 3) {
+				printf("Invalid wep_default_key index %d\n",
+				       bss->ssid.wep.idx);
+				errors++;
+			}
+		} else if (strcmp(buf, "wep_key0") == 0 ||
+			   strcmp(buf, "wep_key1") == 0 ||
+			   strcmp(buf, "wep_key2") == 0 ||
+			   strcmp(buf, "wep_key3") == 0) {
+			if (hostapd_config_read_wep(&bss->ssid.wep,
+						    buf[7] - '0', pos)) {
+				printf("Line %d: invalid WEP key '%s'\n",
+				       line, buf);
+				errors++;
+			}
+		} else if (strcmp(buf, "dynamic_vlan") == 0) {
+			bss->ssid.dynamic_vlan = atoi(pos);
 		} else {
 			printf("Line %d: unknown configuration item '%s'\n",
 			       line, buf);
@@ -1080,26 +1164,27 @@ struct hostapd_config * hostapd_config_read(const char *fname)
 
 	fclose(f);
 
-	if (hostapd_config_read_maclist(accept_mac_file, &conf->accept_mac,
-					&conf->num_accept_mac))
-		errors++;
-	free(accept_mac_file);
-	if (hostapd_config_read_maclist(deny_mac_file, &conf->deny_mac,
-					&conf->num_deny_mac))
-		errors++;
-	free(deny_mac_file);
+	for (i = 0; i < conf->num_bss; i++) {
+		bss = &conf->bss[i];
 
-#ifdef EAP_SERVER
-	if (hostapd_config_read_eap_user(eap_user_file, conf))
-		errors++;
-	free(eap_user_file);
-#endif /* EAP_SERVER */
+		bss->radius->auth_server = bss->radius->auth_servers;
+		bss->radius->acct_server = bss->radius->acct_servers;
 
-	conf->radius->auth_server = conf->radius->auth_servers;
-	conf->radius->acct_server = conf->radius->acct_servers;
+		if (bss->wpa && bss->ieee802_1x) {
+			bss->ssid.security_policy = SECURITY_WPA;
+		} else if (bss->wpa) {
+			bss->ssid.security_policy = SECURITY_WPA_PSK;
+		} else if (bss->ieee802_1x) {
+			bss->ssid.security_policy = SECURITY_IEEE_802_1X;
+			bss->ssid.wep.default_len = bss->default_wep_key_len;
+		} else if (bss->ssid.wep.keys_set)
+			bss->ssid.security_policy = SECURITY_STATIC_WEP;
+		else
+			bss->ssid.security_policy = SECURITY_PLAINTEXT;
 
-	if (hostapd_config_check(conf))
-		errors++;
+		if (hostapd_config_check(bss))
+			errors++;
+	}
 
 	if (errors) {
 		printf("%d errors found in configuration file '%s'\n",
@@ -1132,7 +1217,7 @@ static void hostapd_config_free_eap_user(struct hostapd_eap_user *user)
 }
 
 
-void hostapd_config_free(struct hostapd_config *conf)
+static void hostapd_config_free_bss(struct hostapd_bss_config *conf)
 {
 	struct hostapd_wpa_psk *psk, *prev;
 	struct hostapd_eap_user *user, *prev_user;
@@ -1140,15 +1225,15 @@ void hostapd_config_free(struct hostapd_config *conf)
 	if (conf == NULL)
 		return;
 
-	psk = conf->wpa_psk;
+	psk = conf->ssid.wpa_psk;
 	while (psk) {
 		prev = psk;
 		psk = psk->next;
 		free(prev);
 	}
 
-	free(conf->wpa_passphrase);
-	free(conf->wpa_psk_file);
+	free(conf->ssid.wpa_passphrase);
+	free(conf->ssid.wpa_psk_file);
 
 	user = conf->eap_user;
 	while (user) {
@@ -1175,13 +1260,27 @@ void hostapd_config_free(struct hostapd_config *conf)
 	free(conf->eap_sim_db);
 	free(conf->radius_server_clients);
 	free(conf->test_socket);
+}
+
+
+void hostapd_config_free(struct hostapd_config *conf)
+{
+	int i;
+
+	if (conf == NULL)
+		return;
+
+	for (i = 0; i < conf->num_bss; i++)
+		hostapd_config_free_bss(&conf->bss[i]);
+	free(conf->bss);
+
 	free(conf);
 }
 
 
 /* Perform a binary search for given MAC address from a pre-sorted list.
  * Returns 1 if address is in the list or 0 if not. */
-int hostapd_maclist_found(macaddr *list, int num_entries, u8 *addr)
+int hostapd_maclist_found(macaddr *list, int num_entries, const u8 *addr)
 {
 	int start, end, middle, res;
 
@@ -1203,13 +1302,13 @@ int hostapd_maclist_found(macaddr *list, int num_entries, u8 *addr)
 }
 
 
-const u8 * hostapd_get_psk(const struct hostapd_config *conf, const u8 *addr,
-			   const u8 *prev_psk)
+const u8 * hostapd_get_psk(const struct hostapd_bss_config *conf,
+			   const u8 *addr, const u8 *prev_psk)
 {
 	struct hostapd_wpa_psk *psk;
 	int next_ok = prev_psk == NULL;
 
-	for (psk = conf->wpa_psk; psk != NULL; psk = psk->next) {
+	for (psk = conf->ssid.wpa_psk; psk != NULL; psk = psk->next) {
 		if (next_ok &&
 		    (psk->group || memcmp(psk->addr, addr, ETH_ALEN) == 0))
 			return psk->psk;
@@ -1223,7 +1322,7 @@ const u8 * hostapd_get_psk(const struct hostapd_config *conf, const u8 *addr,
 
 
 const struct hostapd_eap_user *
-hostapd_get_eap_user(const struct hostapd_config *conf, const u8 *identity,
+hostapd_get_eap_user(const struct hostapd_bss_config *conf, const u8 *identity,
 		     size_t identity_len, int phase2)
 {
 	struct hostapd_eap_user *user = conf->eap_user;
@@ -1238,7 +1337,7 @@ hostapd_get_eap_user(const struct hostapd_config *conf, const u8 *identity,
 		    identity_len >= user->identity_len &&
 		    memcmp(user->identity, identity, user->identity_len) == 0)
 		{
-			/* Wildard prefix match */
+			/* Wildcard prefix match */
 			break;
 		}
 
