@@ -1,3 +1,17 @@
+/*
+ * hostapd / Configuration file
+ * Copyright (c) 2003-2005, Jouni Malinen <jkmaline@cc.hut.fi>
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License version 2 as
+ * published by the Free Software Foundation.
+ *
+ * Alternatively, this software may be distributed under the terms of BSD
+ * license.
+ *
+ * See README and COPYING for more details.
+ */
+
 #ifndef CONFIG_H
 #define CONFIG_H
 
@@ -7,6 +21,46 @@ typedef u8 macaddr[ETH_ALEN];
 
 struct hostapd_radius_servers;
 
+#define HOSTAPD_MAX_SSID_LEN 32
+
+#define NUM_WEP_KEYS 4
+struct hostapd_wep_keys {
+	u8 idx;
+	u8 *key[NUM_WEP_KEYS];
+	size_t len[NUM_WEP_KEYS];
+	int keys_set;
+	size_t default_len; /* key length used for dynamic key generation */
+};
+
+typedef enum hostap_security_policy {
+	SECURITY_PLAINTEXT = 0,
+	SECURITY_STATIC_WEP = 1,
+	SECURITY_IEEE_802_1X = 2,
+	SECURITY_WPA_PSK = 3,
+	SECURITY_WPA = 4
+} secpolicy;
+
+struct hostapd_ssid {
+	char ssid[HOSTAPD_MAX_SSID_LEN + 1];
+	size_t ssid_len;
+	int ssid_set;
+
+	char vlan[IFNAMSIZ + 1];
+	secpolicy security_policy;
+
+	struct hostapd_wpa_psk *wpa_psk;
+	char *wpa_passphrase;
+	char *wpa_psk_file;
+
+	struct hostapd_wep_keys wep;
+
+#define DYNAMIC_VLAN_DISABLED 0
+#define DYNAMIC_VLAN_OPTIONAL 1
+#define DYNAMIC_VLAN_REQUIRED 2
+	int dynamic_vlan;
+};
+
+
 #define PMK_LEN 32
 struct hostapd_wpa_psk {
 	struct hostapd_wpa_psk *next;
@@ -20,7 +74,10 @@ struct hostapd_eap_user {
 	struct hostapd_eap_user *next;
 	u8 *identity;
 	size_t identity_len;
-	u8 methods[EAP_USER_MAX_METHODS];
+	struct {
+		int vendor;
+		u32 method;
+	} methods[EAP_USER_MAX_METHODS];
 	u8 *password;
 	size_t password_len;
 	int phase2;
@@ -30,12 +87,13 @@ struct hostapd_eap_user {
 				       * nt_password_hash() */
 };
 
-struct hostapd_config {
+/**
+ * struct hostapd_bss_config - Per-BSS configuration
+ */
+struct hostapd_bss_config {
 	char iface[IFNAMSIZ + 1];
 	char bridge[IFNAMSIZ + 1];
 
-	const struct driver_ops *driver;
-
 	enum {
 		HOSTAPD_LEVEL_DEBUG_VERBOSE = 0,
 		HOSTAPD_LEVEL_DEBUG = 1,
@@ -68,10 +126,8 @@ struct hostapd_config {
 	char *nas_identifier;
 	struct hostapd_radius_servers *radius;
 
-#define HOSTAPD_SSID_LEN 32
-	char ssid[HOSTAPD_SSID_LEN + 1];
-	size_t ssid_len;
-	int ssid_set;
+	struct hostapd_ssid ssid;
+
 	char *eap_req_id_text; /* optional displayable message sent with
 				* EAP Request-Identity */
 	size_t eap_req_id_text_len;
@@ -107,9 +163,6 @@ struct hostapd_config {
 #define HOSTAPD_WPA_VERSION_WPA BIT(0)
 #define HOSTAPD_WPA_VERSION_WPA2 BIT(1)
 	int wpa;
-	struct hostapd_wpa_psk *wpa_psk;
-	char *wpa_passphrase;
-	char *wpa_psk_file;
 #define WPA_KEY_MGMT_IEEE8021X BIT(0)
 #define WPA_KEY_MGMT_PSK BIT(1)
 	int wpa_key_mgmt;
@@ -147,17 +200,30 @@ struct hostapd_config {
 				 * address instead of individual address
 				 * (for driver_wired.c).
 				 */
+
+	int ap_max_inactivity;
+};
+
+
+/**
+ * struct hostapd_config - Per-radio interface configuration
+ */
+struct hostapd_config {
+	struct hostapd_bss_config *bss, *last_bss;
+	size_t num_bss;
+
+	const struct driver_ops *driver;
 };
 
 
 struct hostapd_config * hostapd_config_read(const char *fname);
 void hostapd_config_free(struct hostapd_config *conf);
-int hostapd_maclist_found(macaddr *list, int num_entries, u8 *addr);
-const u8 * hostapd_get_psk(const struct hostapd_config *conf, const u8 *addr,
-			   const u8 *prev_psk);
-int hostapd_setup_wpa_psk(struct hostapd_config *conf);
+int hostapd_maclist_found(macaddr *list, int num_entries, const u8 *addr);
+const u8 * hostapd_get_psk(const struct hostapd_bss_config *conf,
+			   const u8 *addr, const u8 *prev_psk);
+int hostapd_setup_wpa_psk(struct hostapd_bss_config *conf);
 const struct hostapd_eap_user *
-hostapd_get_eap_user(const struct hostapd_config *conf, const u8 *identity,
+hostapd_get_eap_user(const struct hostapd_bss_config *conf, const u8 *identity,
 		     size_t identity_len, int phase2);
 
 #endif /* CONFIG_H */
