@@ -1,6 +1,5 @@
 /*
- * Host AP (software wireless LAN access point) user space daemon for
- * Host AP kernel driver / Kernel driver communication
+ * hostapd / Kernel driver communication with Linux Host AP driver
  * Copyright (c) 2002-2005, Jouni Malinen <jkmaline@cc.hut.fi>
  *
  * This program is free software; you can redistribute it and/or modify
@@ -57,7 +56,8 @@ static int hostapd_ioctl(void *priv, struct prism2_hostapd_param *param,
 			 int len);
 static int hostap_set_iface_flags(void *priv, int dev_up);
 
-static void handle_data(hostapd *hapd, u8 *buf, size_t len, u16 stype)
+static void handle_data(struct hostapd_data *hapd, u8 *buf, size_t len,
+			u16 stype)
 {
 	struct ieee80211_hdr *hdr;
 	u16 fc, ethertype;
@@ -127,7 +127,8 @@ static void handle_data(hostapd *hapd, u8 *buf, size_t len, u16 stype)
 }
 
 
-static void handle_tx_callback(hostapd *hapd, u8 *buf, size_t len, int ok)
+static void handle_tx_callback(struct hostapd_data *hapd, u8 *buf, size_t len,
+			       int ok)
 {
 	struct ieee80211_hdr *hdr;
 	u16 fc, type, stype;
@@ -171,7 +172,7 @@ static void handle_tx_callback(hostapd *hapd, u8 *buf, size_t len, int ok)
 }
 
 
-static void handle_frame(hostapd *hapd, u8 *buf, size_t len)
+static void handle_frame(struct hostapd_data *hapd, u8 *buf, size_t len)
 {
 	struct ieee80211_hdr *hdr;
 	u16 fc, extra_len, type, stype;
@@ -244,7 +245,7 @@ static void handle_frame(hostapd *hapd, u8 *buf, size_t len)
 
 static void handle_read(int sock, void *eloop_ctx, void *sock_ctx)
 {
-	hostapd *hapd = (hostapd *) eloop_ctx;
+	struct hostapd_data *hapd = (struct hostapd_data *) eloop_ctx;
 	int len;
 	unsigned char buf[3000];
 
@@ -460,8 +461,10 @@ static int hostapd_ioctl(void *priv, struct prism2_hostapd_param *param,
 }
 
 
-static int hostap_set_encryption(void *priv, const char *alg, const u8 *addr,
-				 int idx, const u8 *key, size_t key_len)
+static int hostap_set_encryption(const char *ifname, void *priv,
+				 const char *alg, const u8 *addr,
+				 int idx, const u8 *key, size_t key_len,
+				 int txkey)
 {
 	struct hostap_driver_data *drv = priv;
 	struct prism2_hostapd_param *param;
@@ -482,7 +485,7 @@ static int hostap_set_encryption(void *priv, const char *alg, const u8 *addr,
 	else
 		memcpy(param->sta_addr, addr, ETH_ALEN);
 	strncpy((char *) param->u.crypt.alg, alg, HOSTAP_CRYPT_ALG_NAME_LEN);
-	param->u.crypt.flags = HOSTAP_CRYPT_FLAG_SET_TX_KEY;
+	param->u.crypt.flags = txkey ? HOSTAP_CRYPT_FLAG_SET_TX_KEY : 0;
 	param->u.crypt.idx = idx;
 	param->u.crypt.key_len = key_len;
 	memcpy((u8 *) (param + 1), key, key_len);
@@ -497,7 +500,8 @@ static int hostap_set_encryption(void *priv, const char *alg, const u8 *addr,
 }
 
 
-static int hostap_get_seqnum(void *priv, const u8 *addr, int idx, u8 *seq)
+static int hostap_get_seqnum(const char *ifname, void *priv, const u8 *addr,
+			     int idx, u8 *seq)
 {
 	struct hostap_driver_data *drv = priv;
 	struct prism2_hostapd_param *param;
@@ -552,7 +556,7 @@ static int hostap_ioctl_prism2param(void *priv, int param, int value)
 }
 
 
-static int hostap_set_ieee8021x(void *priv, int enabled)
+static int hostap_set_ieee8021x(const char *ifname, void *priv, int enabled)
 {
 	struct hostap_driver_data *drv = priv;
 
@@ -662,8 +666,8 @@ static int hostap_read_sta_data(void *priv,
 }
 
 
-static int hostap_sta_add(void *priv, const u8 *addr, u16 aid, u16 capability,
-			  u8 tx_supp_rates)
+static int hostap_sta_add(const char *ifname, void *priv, const u8 *addr,
+			  u16 aid, u16 capability, u8 tx_supp_rates)
 {
 	struct hostap_driver_data *drv = priv;
 	struct prism2_hostapd_param param;
