@@ -12,6 +12,7 @@ struct driver_ops {
 
 	/**
 	 * set_8021x - enable/disable IEEE 802.1X support
+	 * @ifname: Interface name (for multi-SSID/VLAN support)
 	 * @priv: driver private data
 	 * @enabled: 1 = enable, 0 = disable
 	 *
@@ -20,7 +21,7 @@ struct driver_ops {
 	 * Configure the kernel driver to enable/disable 802.1X support.
 	 * This may be an empty function if 802.1X support is always enabled.
 	 */
-	int (*set_ieee8021x)(void *priv, int enabled);
+	int (*set_ieee8021x)(const char *ifname, void *priv, int enabled);
 
 	/**
 	 * set_privacy - enable/disable privacy
@@ -33,9 +34,11 @@ struct driver_ops {
 	 */
 	int (*set_privacy)(void *priv, int enabled);
 
-	int (*set_encryption)(void *priv, const char *alg, const u8 *addr,
-			      int idx, const u8 *key, size_t key_len);
-	int (*get_seqnum)(void *priv, const u8 *addr, int idx, u8 *seq);
+	int (*set_encryption)(const char *ifname, void *priv, const char *alg,
+			      const u8 *addr, int idx,
+			      const u8 *key, size_t key_len, int txkey);
+	int (*get_seqnum)(const char *ifname, void *priv, const u8 *addr,
+			  int idx, u8 *seq);
 	int (*flush)(void *priv);
 	int (*set_generic_elem)(void *priv, const u8 *elem, size_t elem_len);
 
@@ -54,8 +57,8 @@ struct driver_ops {
 	int (*send_mgmt_frame)(void *priv, const void *msg, size_t len,
 			       int flags);
 	int (*set_assoc_ap)(void *priv, const u8 *addr);
-	int (*sta_add)(void *priv, const u8 *addr, u16 aid, u16 capability,
-		       u8 tx_supp_rates);
+	int (*sta_add)(const char *ifname, void *priv, const u8 *addr, u16 aid,
+		       u16 capability, u8 tx_supp_rates);
 	int (*get_inact_sec)(void *priv, const u8 *addr);
 	int (*sta_clear_stats)(void *priv, const u8 *addr);
 };
@@ -95,11 +98,12 @@ hostapd_wireless_event_deinit(struct hostapd_data *hapd)
 }
 
 static inline int
-hostapd_set_ieee8021x(struct hostapd_data *hapd, int enabled)
+hostapd_set_ieee8021x(const char *ifname, struct hostapd_data *hapd,
+		      int enabled)
 {
 	if (hapd->driver == NULL || hapd->driver->set_ieee8021x == NULL)
 		return 0;
-	return hapd->driver->set_ieee8021x(hapd->driver, enabled);
+	return hapd->driver->set_ieee8021x(ifname, hapd->driver, enabled);
 }
 
 static inline int
@@ -111,21 +115,23 @@ hostapd_set_privacy(struct hostapd_data *hapd, int enabled)
 }
 
 static inline int
-hostapd_set_encryption(struct hostapd_data *hapd, const char *alg,
-		       const u8 *addr, int idx, u8 *key, size_t key_len)
+hostapd_set_encryption(const char *ifname, struct hostapd_data *hapd,
+		       const char *alg, const u8 *addr, int idx,
+		       u8 *key, size_t key_len, int txkey)
 {
 	if (hapd->driver == NULL || hapd->driver->set_encryption == NULL)
 		return 0;
-	return hapd->driver->set_encryption(hapd->driver, alg, addr, idx, key,
-					    key_len);
+	return hapd->driver->set_encryption(ifname, hapd->driver, alg, addr,
+					    idx, key, key_len, txkey);
 }
 
 static inline int
-hostapd_get_seqnum(struct hostapd_data *hapd, const u8 *addr, int idx, u8 *seq)
+hostapd_get_seqnum(const char *ifname, struct hostapd_data *hapd,
+		   const u8 *addr, int idx, u8 *seq)
 {
 	if (hapd->driver == NULL || hapd->driver->get_seqnum == NULL)
 		return 0;
-	return hapd->driver->get_seqnum(hapd->driver, addr, idx, seq);
+	return hapd->driver->get_seqnum(ifname, hapd->driver, addr, idx, seq);
 }
 
 static inline int
@@ -240,13 +246,13 @@ hostapd_set_countermeasures(struct hostapd_data *hapd, int enabled)
 }
 
 static inline int
-hostapd_sta_add(struct hostapd_data *hapd, const u8 *addr, u16 aid,
-		u16 capability, u8 tx_supp_rates)
+hostapd_sta_add(const char *ifname, struct hostapd_data *hapd, const u8 *addr,
+		u16 aid, u16 capability, u8 tx_supp_rates)
 {
 	if (hapd->driver == NULL || hapd->driver->sta_add == NULL)
 		return 0;
-	return hapd->driver->sta_add(hapd->driver, addr, aid, capability,
-				     tx_supp_rates);
+	return hapd->driver->sta_add(ifname, hapd->driver, addr, aid,
+				     capability, tx_supp_rates);
 }
 
 static inline int
