@@ -1,5 +1,5 @@
 /*
- * Host AP - driver interaction with MADWIFI 802.11 driver
+ * hostapd / Driver interaction with MADWIFI 802.11 driver
  * Copyright (c) 2004, Sam Leffler <sam@errno.com>
  * Copyright (c) 2004, Video54 Technologies
  * Copyright (c) 2005, Jouni Malinen <jkmaline@cc.hut.fi>
@@ -99,26 +99,39 @@ set80211priv(struct madwifi_driver_data *drv, int op, void *data, int len)
 	if (ioctl(drv->ioctl_sock, op, &iwr) < 0) {
 #ifdef MADWIFI_NG
 		int first = IEEE80211_IOCTL_SETPARAM;
-		int last = IEEE80211_IOCTL_GETMODE;
+		int last = IEEE80211_IOCTL_KICKMAC;
 		static const char *opnames[] = {
 			"ioctl[IEEE80211_IOCTL_SETPARAM]",
 			"ioctl[IEEE80211_IOCTL_GETPARAM]",
-			"ioctl[IEEE80211_IOCTL_SETKEY]",
+			"ioctl[IEEE80211_IOCTL_SETMODE]",
+			"ioctl[IEEE80211_IOCTL_GETMODE]",
 			"ioctl[IEEE80211_IOCTL_SETWMMPARAMS]",
-			"ioctl[IEEE80211_IOCTL_DELKEY]",
 			"ioctl[IEEE80211_IOCTL_GETWMMPARAMS]",
-			"ioctl[IEEE80211_IOCTL_SETMLME]",
+			"ioctl[IEEE80211_IOCTL_SETCHANLIST]",
+			"ioctl[IEEE80211_IOCTL_GETCHANLIST]",
+			"ioctl[IEEE80211_IOCTL_CHANSWITCH]",
+			NULL,
+			NULL,
+			"ioctl[IEEE80211_IOCTL_GETSCANRESULTS]",
+			NULL,
 			"ioctl[IEEE80211_IOCTL_GETCHANINFO]",
 			"ioctl[IEEE80211_IOCTL_SETOPTIE]",
 			"ioctl[IEEE80211_IOCTL_GETOPTIE]",
+			"ioctl[IEEE80211_IOCTL_SETMLME]",
+			NULL,
+			"ioctl[IEEE80211_IOCTL_SETKEY]",
+			NULL,
+			"ioctl[IEEE80211_IOCTL_DELKEY]",
+			NULL,
 			"ioctl[IEEE80211_IOCTL_ADDMAC]",
-			"ioctl[SIOCIWFIRSTPRIV+11]",
+			NULL,
 			"ioctl[IEEE80211_IOCTL_DELMAC]",
-			"ioctl[IEEE80211_IOCTL_GETCHANLIST]",
-			"ioctl[IEEE80211_IOCTL_SETCHANLIST]",
-			"ioctl[SIOCIWFIRSTPRIV+15]",
-			"ioctl[IEEE80211_IOCTL_SETMODE]",
-			"ioctl[IEEE80211_IOCTL_GETMODE]",
+			NULL,
+			"ioctl[IEEE80211_IOCTL_WDSMAC]",
+			NULL,
+			"ioctl[IEEE80211_IOCTL_WDSDELMAC]",
+			NULL,
+			"ioctl[IEEE80211_IOCTL_KICMAC]",
 		};
 #else /* MADWIFI_NG */
 		int first = IEEE80211_IOCTL_SETPARAM;
@@ -145,8 +158,10 @@ set80211priv(struct madwifi_driver_data *drv, int op, void *data, int len)
 			"ioctl[IEEE80211_IOCTL_GETKEY]",
 		};
 #endif /* MADWIFI_NG */
-		if (first <= op && op <= last)
-			perror(opnames[op - first]);
+		int idx = op - first;
+		if (first <= op && op <= last &&
+		    idx < sizeof(opnames) / sizeof(opnames[0]) && opnames[idx])
+			perror(opnames[idx]);
 		else
 			perror("ioctl[unknown???]");
 		return -1;
@@ -189,8 +204,8 @@ ether_sprintf(const u8 *addr)
 static int
 madwifi_configure_wpa(struct madwifi_driver_data *drv)
 {
-	hostapd *hapd = drv->hapd;
-	struct hostapd_config *conf = hapd->conf;
+	struct hostapd_data *hapd = drv->hapd;
+	struct hostapd_bss_config *conf = hapd->conf;
 	int v;
 
 	switch (conf->wpa_group) {
@@ -276,7 +291,7 @@ static int
 madwifi_set_iface_flags(void *priv, int dev_up)
 {
 	struct madwifi_driver_data *drv = priv;
-	hostapd *hapd = drv->hapd;
+	struct hostapd_data *hapd = drv->hapd;
 	struct ifreq ifr;
 
 	HOSTAPD_DEBUG(HOSTAPD_DEBUG_VERBOSE,
@@ -318,11 +333,11 @@ madwifi_set_iface_flags(void *priv, int dev_up)
 }
 
 static int
-madwifi_set_ieee8021x(void *priv, int enabled)
+madwifi_set_ieee8021x(const char *ifname, void *priv, int enabled)
 {
 	struct madwifi_driver_data *drv = priv;
-	hostapd *hapd = drv->hapd;
-	struct hostapd_config *conf = hapd->conf;
+	struct hostapd_data *hapd = drv->hapd;
+	struct hostapd_bss_config *conf = hapd->conf;
 
 	HOSTAPD_DEBUG(HOSTAPD_DEBUG_VERBOSE,
 		"%s: enabled=%d\n", __func__, enabled);
@@ -355,7 +370,7 @@ static int
 madwifi_set_privacy(void *priv, int enabled)
 {
 	struct madwifi_driver_data *drv = priv;
-	hostapd *hapd = drv->hapd;
+	struct hostapd_data *hapd = drv->hapd;
 
 	HOSTAPD_DEBUG(HOSTAPD_DEBUG_MINIMAL,
 		"%s: enabled=%d\n", __func__, enabled);
@@ -367,7 +382,7 @@ static int
 madwifi_set_sta_authorized(void *priv, const u8 *addr, int authorized)
 {
 	struct madwifi_driver_data *drv = priv;
-	hostapd *hapd = drv->hapd;
+	struct hostapd_data *hapd = drv->hapd;
 	struct ieee80211req_mlme mlme;
 
 	HOSTAPD_DEBUG(HOSTAPD_DEBUG_VERBOSE,
@@ -388,7 +403,7 @@ static int
 madwifi_del_key(void *priv, const u8 *addr, int key_idx)
 {
 	struct madwifi_driver_data *drv = priv;
-	hostapd *hapd = drv->hapd;
+	struct hostapd_data *hapd = drv->hapd;
 	struct ieee80211req_del_key wk;
 
 	HOSTAPD_DEBUG(HOSTAPD_DEBUG_MINIMAL,
@@ -407,12 +422,12 @@ madwifi_del_key(void *priv, const u8 *addr, int key_idx)
 }
 
 static int
-madwifi_set_key(void *priv, const char *alg,
-	     const u8 *addr, int key_idx,
-	     const u8 *key, size_t key_len)
+madwifi_set_key(const char *ifname, void *priv, const char *alg,
+		const u8 *addr, int key_idx,
+		const u8 *key, size_t key_len, int txkey)
 {
 	struct madwifi_driver_data *drv = priv;
-	hostapd *hapd = drv->hapd;
+	struct hostapd_data *hapd = drv->hapd;
 	struct ieee80211req_key wk;
 	u_int8_t cipher;
 
@@ -460,10 +475,11 @@ madwifi_set_key(void *priv, const char *alg,
 
 
 static int
-madwifi_get_seqnum(void *priv, const u8 *addr, int idx, u8 *seq)
+madwifi_get_seqnum(const char *ifname, void *priv, const u8 *addr, int idx,
+		   u8 *seq)
 {
 	struct madwifi_driver_data *drv = priv;
-	hostapd *hapd = drv->hapd;
+	struct hostapd_data *hapd = drv->hapd;
 	struct ieee80211req_key wk;
 
 	HOSTAPD_DEBUG(HOSTAPD_DEBUG_MINIMAL,
@@ -631,7 +647,7 @@ static int
 madwifi_sta_deauth(void *priv, const u8 *addr, int reason_code)
 {
 	struct madwifi_driver_data *drv = priv;
-	hostapd *hapd = drv->hapd;
+	struct hostapd_data *hapd = drv->hapd;
 	struct ieee80211req_mlme mlme;
 
 	HOSTAPD_DEBUG(HOSTAPD_DEBUG_MINIMAL,
@@ -648,7 +664,7 @@ static int
 madwifi_sta_disassoc(void *priv, const u8 *addr, int reason_code)
 {
 	struct madwifi_driver_data *drv = priv;
-	hostapd *hapd = drv->hapd;
+	struct hostapd_data *hapd = drv->hapd;
 	struct ieee80211req_mlme mlme;
 
 	HOSTAPD_DEBUG(HOSTAPD_DEBUG_MINIMAL,
@@ -667,7 +683,7 @@ madwifi_del_sta(struct madwifi_driver_data *drv, u8 addr[IEEE80211_ADDR_LEN])
 	struct sta_info *sta;
 
 	hostapd_logger(hapd, addr, HOSTAPD_MODULE_IEEE80211,
-		HOSTAPD_LEVEL_INFO, "deassociated");
+		HOSTAPD_LEVEL_INFO, "disassociated");
 
 	sta = ap_get_sta(hapd, addr);
 	if (sta != NULL) {
@@ -686,6 +702,7 @@ madwifi_process_wpa_ie(struct madwifi_driver_data *drv, struct sta_info *sta)
 	struct hostapd_data *hapd = drv->hapd;
 	struct ieee80211req_wpaie ie;
 	int ielen, res;
+	u8 *iebuf;
 
 	/*
 	 * Fetch negotiated WPA/RSN parameters from the system.
@@ -696,7 +713,15 @@ madwifi_process_wpa_ie(struct madwifi_driver_data *drv, struct sta_info *sta)
 		printf("Failed to get WPA/RSN information element.\n");
 		return -1;		/* XXX not right */
 	}
-	ielen = ie.wpa_ie[1];
+	iebuf = ie.wpa_ie;
+#ifdef MADWIFI_NG
+	if (iebuf[1] == 0 && ie.rsn_ie[1] > 0) {
+		/* madwifi-ng svn #1453 added rsn_ie. Use it, if wpa_ie was not
+		 * set. This is needed for WPA2. */
+		iebuf = ie.rsn_ie;
+	}
+#endif /* MADWIFI_NG */
+	ielen = iebuf[1];
 	if (ielen == 0) {
 		printf("No WPA/RSN information element for station!?\n");
 		return -1;		/* XXX not right */
@@ -709,7 +734,7 @@ madwifi_process_wpa_ie(struct madwifi_driver_data *drv, struct sta_info *sta)
 		return -1;
 	}
 	res = wpa_validate_wpa_ie(hapd->wpa_auth, sta->wpa_sm,
-				  ie.wpa_ie, ielen);
+				  iebuf, ielen);
 	if (res != WPA_IE_OK) {
 		printf("WPA/RSN information element rejected? (res %u)\n", res);
 		return -1;
@@ -1063,7 +1088,7 @@ madwifi_send_eapol(void *priv, const u8 *addr, const u8 *data, size_t data_len,
 	int status;
 
 	/*
-	 * Prepend the Etherent header.  If the caller left us
+	 * Prepend the Ethernet header.  If the caller left us
 	 * space at the front we could just insert it but since
 	 * we don't know we copy to a local buffer.  Given the frequency
 	 * and size of frames this probably doesn't matter.
@@ -1096,7 +1121,7 @@ static void
 handle_read(void *ctx, const u8 *src_addr, const u8 *buf, size_t len)
 {
 	struct madwifi_driver_data *drv = ctx;
-	hostapd *hapd = drv->hapd;
+	struct hostapd_data *hapd = drv->hapd;
 	struct sta_info *sta;
 
 	sta = ap_get_sta(hapd, src_addr);
