@@ -1,5 +1,5 @@
 /*
- * Host AP - driver interaction with Prism54 PIMFOR interface
+ * hostapd / Driver interaction with Prism54 PIMFOR interface
  * Copyright (c) 2004, Bell Kin <bell_kin@pek.com.tw>
  * based on hostap driver.c, ieee802_11.c
  *
@@ -192,8 +192,10 @@ static int prism54_set_sta_authorized(void *priv, const u8 *addr,
 
 
 /* set per station key */
-static int prism54_set_encryption(void *priv, const char *alg, const u8 *addr,
-				  int idx, const u8 *key, size_t key_len)
+static int prism54_set_encryption(const char *ifname, void *priv,
+				  const char *alg, const u8 *addr,
+				  int idx, const u8 *key, size_t key_len,
+				  int txkey)
 {
 	struct prism54_driver_data *drv = priv;
 	pimdev_hdr *hdr;
@@ -249,7 +251,8 @@ static int prism54_set_encryption(void *priv, const char *alg, const u8 *addr,
 
 
 /* get TKIP station sequence counter, prism54 is only 6 bytes */
-static int prism54_get_seqnum(void *priv, const u8 *addr, int idx, u8 *seq)
+static int prism54_get_seqnum(const char *ifname, void *priv, const u8 *addr,
+			      int idx, u8 *seq)
 {
 	struct prism54_driver_data *drv = priv;
 	struct obj_stasc *stasc;
