@@ -1,7 +1,6 @@
 /*
- * Host AP (software wireless LAN access point) user space daemon for
- * Host AP kernel driver / Driver interface for development testing
- * Copyright (c) 2004-2005, Jouni Malinen <jkmaline@cc.hut.fi>
+ * hostapd / Driver interface for development testing
+ * Copyright (c) 2004-2006, Jouni Malinen <jkmaline@cc.hut.fi>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 2 as
@@ -45,11 +44,23 @@ struct test_driver_data {
 	int test_socket;
 	u8 *ie;
 	size_t ielen;
+	char *own_socket_path;
 };
 
 static const struct driver_ops test_driver_ops;
 
 
+static void test_driver_free_priv(struct test_driver_data *drv)
+{
+	if (drv == NULL)
+		return;
+
+	free(drv->ie);
+	free(drv->own_socket_path);
+	free(drv);
+}
+
+
 static struct test_client_socket *
 test_driver_get_cli(struct test_driver_data *drv, struct sockaddr_un *from,
 		    socklen_t fromlen)
@@ -58,7 +69,8 @@ test_driver_get_cli(struct test_driver_data *drv, struct sockaddr_un *from,
 
 	while (cli) {
 		if (cli->unlen == fromlen &&
-		    strncmp(cli->un.sun_path, from->sun_path, fromlen) == 0)
+		    strncmp(cli->un.sun_path, from->sun_path,
+			    fromlen - sizeof(cli->un.sun_family)) == 0)
 			return cli;
 		cli = cli->next;
 	}
@@ -113,7 +125,6 @@ static void test_driver_scan(struct test_driver_data *drv,
 			     struct sockaddr_un *from, socklen_t fromlen)
 {
 	char buf[512], *pos, *end;
-	int i;
 
 	pos = buf;
 	end = buf + sizeof(buf);
@@ -123,15 +134,10 @@ static void test_driver_scan(struct test_driver_data *drv,
 	/* reply: SCANRESP BSSID SSID IEs */
 	pos += snprintf(pos, end - pos, "SCANRESP " MACSTR " ",
 			MAC2STR(drv->hapd->own_addr));
-	for (i = 0; i < drv->hapd->conf->ssid_len; i++) {
-		pos += snprintf(pos, end - pos, "%02x",
-				drv->hapd->conf->ssid[i]);
-	}
+	pos += wpa_snprintf_hex(pos, end - pos, drv->hapd->conf->ssid.ssid,
+				drv->hapd->conf->ssid.ssid_len);
 	pos += snprintf(pos, end - pos, " ");
-	for (i = 0; i < drv->ielen; i++) {
-		pos += snprintf(pos, end - pos, "%02x",
-				drv->ie[i]);
-	}
+	pos += wpa_snprintf_hex(pos, end - pos, drv->ie, drv->ielen);
 
 	sendto(drv->test_socket, buf, pos - buf, 0,
 	       (struct sockaddr *) from, fromlen);
@@ -235,7 +241,8 @@ static void test_driver_assoc(struct test_driver_data *drv,
 	cli->next = drv->cli;
 	drv->cli = cli;
 	wpa_hexdump_ascii(MSG_DEBUG, "test_socket: ASSOC sun_path",
-			  cli->un.sun_path, cli->unlen);
+			  cli->un.sun_path,
+			  cli->unlen - sizeof(cli->un.sun_family));
 
 	snprintf(cmd, sizeof(cmd), "ASSOCRESP " MACSTR " 0",
 		 MAC2STR(drv->hapd->own_addr));
@@ -409,32 +416,49 @@ static int test_driver_init(struct hostapd_data *hapd)
 	hapd->own_addr[0] = 0x02; /* locally administered */
 	sha1_prf(hapd->conf->iface, strlen(hapd->conf->iface),
 		 "hostapd test bssid generation",
-		 hapd->conf->ssid, hapd->conf->ssid_len,
+		 hapd->conf->ssid.ssid, hapd->conf->ssid.ssid_len,
 		 hapd->own_addr + 1, ETH_ALEN - 1);
 
 	if (hapd->conf->test_socket) {
 		if (strlen(hapd->conf->test_socket) >= sizeof(addr.sun_path)) {
 			printf("Too long test_socket path\n");
-			free(drv);
+			test_driver_free_priv(drv);
 			return -1;
 		}
+		if (strncmp(hapd->conf->test_socket, "DIR:", 4) == 0) {
+			size_t len = strlen(hapd->conf->test_socket) + 30;
+			drv->own_socket_path = malloc(len);
+			if (drv->own_socket_path) {
+				snprintf(drv->own_socket_path, len,
+					 "%s/AP-" MACSTR,
+					 hapd->conf->test_socket + 4,
+					 MAC2STR(hapd->own_addr));
+			}
+		} else {
+			drv->own_socket_path = strdup(hapd->conf->test_socket);
+		}
+		if (drv->own_socket_path == NULL) {
+			test_driver_free_priv(drv);
+			return -1;
+		}
+
 		drv->test_socket = socket(PF_UNIX, SOCK_DGRAM, 0);
 		if (drv->test_socket < 0) {
 			perror("socket(PF_UNIX)");
-			free(drv);
+			test_driver_free_priv(drv);
 			return -1;
 		}
 
 		memset(&addr, 0, sizeof(addr));
 		addr.sun_family = AF_UNIX;
-		strncpy(addr.sun_path, hapd->conf->test_socket,
+		strncpy(addr.sun_path, drv->own_socket_path,
 			sizeof(addr.sun_path));
 		if (bind(drv->test_socket, (struct sockaddr *) &addr,
 			 sizeof(addr)) < 0) {
 			perror("bind(PF_UNIX)");
 			close(drv->test_socket);
-			unlink(hapd->conf->test_socket);
-			free(drv);
+			unlink(drv->own_socket_path);
+			test_driver_free_priv(drv);
 			return -1;
 		}
 		eloop_register_read_sock(drv->test_socket,
@@ -462,13 +486,12 @@ static void test_driver_deinit(void *priv)
 	if (drv->test_socket >= 0) {
 		eloop_unregister_read_sock(drv->test_socket);
 		close(drv->test_socket);
-		unlink(drv->hapd->conf->test_socket);
+		unlink(drv->own_socket_path);
 	}
 
 	drv->hapd->driver = NULL;
 
-	free(drv->ie);
-	free(drv);
+	test_driver_free_priv(drv);
 }
 
 
