@@ -1,6 +1,6 @@
 /*
- * hostapd / EAP-AKA (draft-arkko-pppext-eap-aka-15.txt)
- * Copyright (c) 2005, Jouni Malinen <jkmaline@cc.hut.fi>
+ * hostapd / EAP-AKA (RFC 4187)
+ * Copyright (c) 2005-2006, Jouni Malinen <jkmaline@cc.hut.fi>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 2 as
@@ -39,6 +39,8 @@ struct eap_aka_data {
 	char *next_reauth_id;
 	u16 counter;
 	struct eap_sim_reauth *reauth;
+	int auts_reported; /* whether the current AUTS has been reported to the
+			    * eap_sim_db */
 };
 
 
@@ -247,6 +249,7 @@ static u8 * eap_aka_buildReq(struct eap_sm *sm, void *priv, int id,
 {
 	struct eap_aka_data *data = priv;
 
+	data->auts_reported = 0;
 	switch (data->state) {
 	case IDENTITY:
 		return eap_aka_build_identity(sm, data, id, reqDataLen);
@@ -280,7 +283,8 @@ static Boolean eap_aka_check(struct eap_sm *sm, void *priv,
 	}
 	subtype = pos[1];
 
-	if (subtype == EAP_AKA_SUBTYPE_CLIENT_ERROR)
+	if (subtype == EAP_AKA_SUBTYPE_CLIENT_ERROR ||
+	    subtype == EAP_AKA_SUBTYPE_AUTHENTICATION_REJECT)
 		return FALSE;
 
 	switch (data->state) {
@@ -359,12 +363,14 @@ static void eap_aka_determine_identity(struct eap_sm *sm,
 			wpa_printf(MSG_DEBUG, "EAP-AKA: Permanent user name "
 				   "not known - send AKA-Identity request");
 			eap_aka_state(data, IDENTITY);
+			return;
 		} else {
-			wpa_printf(MSG_DEBUG, "EAP-AKA: Could not get proper "
-				   "permanent user name");
-			eap_aka_state(data, FAILURE);
+			wpa_printf(MSG_DEBUG, "EAP-AKA: Unknown whether the "
+				   "permanent user name is known; try to use "
+				   "it");
+			/* eap_sim_db_get_aka_auth() will report failure, if
+			 * this identity is not known. */
 		}
-		return;
 	}
 
 	wpa_hexdump_ascii(MSG_DEBUG, "EAP-AKA: Identity",
@@ -507,12 +513,18 @@ static void eap_aka_process_sync_failure(struct eap_sm *sm,
 		return;
 	}
 
-	if (eap_sim_db_resynchronize(sm->eap_sim_db_priv, sm->identity,
-				     sm->identity_len, attr->auts)) {
+	/* Avoid re-reporting AUTS when processing pending EAP packet by
+	 * maintaining a local flag stating whether this AUTS has already been
+	 * reported. */
+	if (!data->auts_reported &&
+	    eap_sim_db_resynchronize(sm->eap_sim_db_priv, sm->identity,
+				     sm->identity_len, attr->auts,
+				     data->rand)) {
 		wpa_printf(MSG_WARNING, "EAP-AKA: Resynchronization failed");
 		eap_aka_state(data, FAILURE);
 		return;
 	}
+	data->auts_reported = 1;
 
 	/* Try again after resynchronization */
 	eap_aka_determine_identity(sm, data, 0);
@@ -699,16 +711,27 @@ static Boolean eap_aka_isSuccess(struct eap_sm *sm, void *priv)
 }
 
 
-const struct eap_method eap_method_aka =
+int eap_server_aka_register(void)
 {
-	.method = EAP_TYPE_AKA,
-	.name = "AKA",
-	.init = eap_aka_init,
-	.reset = eap_aka_reset,
-	.buildReq = eap_aka_buildReq,
-	.check = eap_aka_check,
-	.process = eap_aka_process,
-	.isDone = eap_aka_isDone,
-	.getKey = eap_aka_getKey,
-	.isSuccess = eap_aka_isSuccess,
-};
+	struct eap_method *eap;
+	int ret;
+
+	eap = eap_server_method_alloc(EAP_SERVER_METHOD_INTERFACE_VERSION,
+				      EAP_VENDOR_IETF, EAP_TYPE_AKA, "AKA");
+	if (eap == NULL)
+		return -1;
+
+	eap->init = eap_aka_init;
+	eap->reset = eap_aka_reset;
+	eap->buildReq = eap_aka_buildReq;
+	eap->check = eap_aka_check;
+	eap->process = eap_aka_process;
+	eap->isDone = eap_aka_isDone;
+	eap->getKey = eap_aka_getKey;
+	eap->isSuccess = eap_aka_isSuccess;
+
+	ret = eap_server_method_register(eap);
+	if (ret)
+		eap_server_method_free(eap);
+	return ret;
+}
