@@ -169,15 +169,26 @@ static Boolean eap_md5_isSuccess(struct eap_sm *sm, void *priv)
 }
 
 
-const struct eap_method eap_method_md5 =
+int eap_server_md5_register(void)
 {
-	.method = EAP_TYPE_MD5,
-	.name = "MD5",
-	.init = eap_md5_init,
-	.reset = eap_md5_reset,
-	.buildReq = eap_md5_buildReq,
-	.check = eap_md5_check,
-	.process = eap_md5_process,
-	.isDone = eap_md5_isDone,
-	.isSuccess = eap_md5_isSuccess,
-};
+	struct eap_method *eap;
+	int ret;
+
+	eap = eap_server_method_alloc(EAP_SERVER_METHOD_INTERFACE_VERSION,
+				      EAP_VENDOR_IETF, EAP_TYPE_MD5, "MD5");
+	if (eap == NULL)
+		return -1;
+
+	eap->init = eap_md5_init;
+	eap->reset = eap_md5_reset;
+	eap->buildReq = eap_md5_buildReq;
+	eap->check = eap_md5_check;
+	eap->process = eap_md5_process;
+	eap->isDone = eap_md5_isDone;
+	eap->isSuccess = eap_md5_isSuccess;
+
+	ret = eap_server_method_register(eap);
+	if (ret)
+		eap_server_method_free(eap);
+	return ret;
+}
