@@ -48,6 +48,7 @@ struct eap_mschapv2_hdr {
 #define ERROR_CHANGING_PASSWORD 709
 
 #define PASSWD_CHANGE_CHAL_LEN 16
+#define MSCHAPV2_KEY_LEN 16
 
 
 #define CHALLENGE_LEN 16
@@ -57,6 +58,8 @@ struct eap_mschapv2_data {
 	u8 auth_response[20];
 	enum { CHALLENGE, SUCCESS_REQ, FAILURE_REQ, SUCCESS, FAILURE } state;
 	u8 resp_mschapv2_id;
+	u8 master_key[16];
+	int master_key_valid;
 };
 
 
@@ -133,7 +136,6 @@ static u8 * eap_mschapv2_build_success_req(struct eap_sm *sm,
 	u8 *pos, *msg, *end;
 	char *message = "OK";
 	size_t msg_len;
-	int i;
 
 	msg_len = 2 + 2 * sizeof(data->auth_response) + 3 + strlen(message);
 	*reqDataLen = sizeof(*req) + msg_len;
@@ -158,10 +160,9 @@ static u8 * eap_mschapv2_build_success_req(struct eap_sm *sm,
 	end = ((u8 *) req) + *reqDataLen + 1;
 
 	pos += snprintf((char *) pos, end - pos, "S=");
-	for (i = 0; i < sizeof(data->auth_response); i++) {
-		pos += snprintf((char *) pos, end - pos, "%02X",
-				data->auth_response[i]);
-	}
+	pos += wpa_snprintf_hex_uppercase((char *) pos, end - pos,
+					  data->auth_response,
+					  sizeof(data->auth_response));
 	pos += snprintf((char *) pos, end - pos, " M=%s", message);
 
 	wpa_hexdump_ascii(MSG_MSGDUMP, "EAP-MSCHAPV2: Success Request Message",
@@ -367,19 +368,26 @@ static void eap_mschapv2_process_response(struct eap_sm *sm,
 	}
 
 	if (memcmp(nt_response, expected, 24) == 0) {
+		const u8 *pw_hash;
+		u8 pw_hash_buf[16], pw_hash_hash[16];
+
 		wpa_printf(MSG_DEBUG, "EAP-MSCHAPV2: Correct NT-Response");
 		data->state = SUCCESS_REQ;
 
-
 		/* Authenticator response is not really needed yet, but
 		 * calculate it here so that peer_challenge and username need
 		 * not be saved. */
 		if (sm->user->password_hash) {
+			pw_hash = sm->user->password;
 			generate_authenticator_response_pwhash(
 				sm->user->password, peer_challenge,
 				data->auth_challenge, username, username_len,
 				nt_response, data->auth_response);
 		} else {
+			nt_password_hash(sm->user->password,
+					 sm->user->password_len,
+					 pw_hash_buf);
+			pw_hash = pw_hash_buf;
 			generate_authenticator_response(sm->user->password,
 							sm->user->password_len,
 							peer_challenge,
@@ -388,6 +396,12 @@ static void eap_mschapv2_process_response(struct eap_sm *sm,
 							nt_response,
 							data->auth_response);
 		}
+
+		hash_nt_password_hash(pw_hash, pw_hash_hash);
+		get_master_key(pw_hash_hash, nt_response, data->master_key);
+		data->master_key_valid = 1;
+		wpa_hexdump_key(MSG_DEBUG, "EAP-MSCHAPV2: Derived Master Key",
+				data->master_key, MSCHAPV2_KEY_LEN);
 	} else {
 		wpa_hexdump(MSG_MSGDUMP, "EAP-MSCHAPV2: Expected NT-Response",
 			    expected, 24);
@@ -475,6 +489,27 @@ static Boolean eap_mschapv2_isDone(struct eap_sm *sm, void *priv)
 }
 
 
+static u8 * eap_mschapv2_getKey(struct eap_sm *sm, void *priv, size_t *len)
+{
+	struct eap_mschapv2_data *data = priv;
+	u8 *key;
+
+	if (data->state != SUCCESS || !data->master_key_valid)
+		return NULL;
+
+	*len = 2 * MSCHAPV2_KEY_LEN;
+	key = malloc(*len);
+	if (key == NULL)
+		return NULL;
+	get_asymetric_start_key(data->master_key, key, MSCHAPV2_KEY_LEN, 0, 0);
+	get_asymetric_start_key(data->master_key, key + MSCHAPV2_KEY_LEN,
+				MSCHAPV2_KEY_LEN, 1, 0);
+	wpa_hexdump_key(MSG_DEBUG, "EAP-MSCHAPV2: Derived key", key, *len);
+
+	return key;
+}
+
+
 static Boolean eap_mschapv2_isSuccess(struct eap_sm *sm, void *priv)
 {
 	struct eap_mschapv2_data *data = priv;
@@ -482,15 +517,28 @@ static Boolean eap_mschapv2_isSuccess(struct eap_sm *sm, void *priv)
 }
 
 
-const struct eap_method eap_method_mschapv2 =
+int eap_server_mschapv2_register(void)
 {
-	.method = EAP_TYPE_MSCHAPV2,
-	.name = "MSCHAPV2",
-	.init = eap_mschapv2_init,
-	.reset = eap_mschapv2_reset,
-	.buildReq = eap_mschapv2_buildReq,
-	.check = eap_mschapv2_check,
-	.process = eap_mschapv2_process,
-	.isDone = eap_mschapv2_isDone,
-	.isSuccess = eap_mschapv2_isSuccess,
-};
+	struct eap_method *eap;
+	int ret;
+
+	eap = eap_server_method_alloc(EAP_SERVER_METHOD_INTERFACE_VERSION,
+				      EAP_VENDOR_IETF, EAP_TYPE_MSCHAPV2,
+				      "MSCHAPV2");
+	if (eap == NULL)
+		return -1;
+
+	eap->init = eap_mschapv2_init;
+	eap->reset = eap_mschapv2_reset;
+	eap->buildReq = eap_mschapv2_buildReq;
+	eap->check = eap_mschapv2_check;
+	eap->process = eap_mschapv2_process;
+	eap->isDone = eap_mschapv2_isDone;
+	eap->getKey = eap_mschapv2_getKey;
+	eap->isSuccess = eap_mschapv2_isSuccess;
+
+	ret = eap_server_method_register(eap);
+	if (ret)
+		eap_server_method_free(eap);
+	return ret;
+}
