@@ -251,13 +251,17 @@ static void eap_psk_process_2(struct eap_sm *sm,
 	}
 
 	for (i = 0;
-	     i < EAP_MAX_METHODS && sm->user->methods[i] != EAP_TYPE_NONE;
+	     i < EAP_MAX_METHODS &&
+		     (sm->user->methods[i].vendor != EAP_VENDOR_IETF ||
+		      sm->user->methods[i].method != EAP_TYPE_NONE);
 	     i++) {
-		if (sm->user->methods[i] == EAP_TYPE_PSK)
+		if (sm->user->methods[i].vendor == EAP_VENDOR_IETF &&
+		    sm->user->methods[i].method == EAP_TYPE_PSK)
 			break;
 	}
 
-	if (sm->user->methods[i] != EAP_TYPE_PSK) {
+	if (sm->user->methods[i].vendor != EAP_VENDOR_IETF ||
+	    sm->user->methods[i].method != EAP_TYPE_PSK) {
 		wpa_hexdump_ascii(MSG_DEBUG,
 				  "EAP-PSK: EAP-PSK not enabled for ID_P",
 				  data->id_p, data->id_p_len);
@@ -442,16 +446,27 @@ static Boolean eap_psk_isSuccess(struct eap_sm *sm, void *priv)
 }
 
 
-const struct eap_method eap_method_psk =
+int eap_server_psk_register(void)
 {
-	.method = EAP_TYPE_PSK,
-	.name = "PSK",
-	.init = eap_psk_init,
-	.reset = eap_psk_reset,
-	.buildReq = eap_psk_buildReq,
-	.check = eap_psk_check,
-	.process = eap_psk_process,
-	.isDone = eap_psk_isDone,
-	.getKey = eap_psk_getKey,
-	.isSuccess = eap_psk_isSuccess,
-};
+	struct eap_method *eap;
+	int ret;
+
+	eap = eap_server_method_alloc(EAP_SERVER_METHOD_INTERFACE_VERSION,
+				      EAP_VENDOR_IETF, EAP_TYPE_PSK, "PSK");
+	if (eap == NULL)
+		return -1;
+
+	eap->init = eap_psk_init;
+	eap->reset = eap_psk_reset;
+	eap->buildReq = eap_psk_buildReq;
+	eap->check = eap_psk_check;
+	eap->process = eap_psk_process;
+	eap->isDone = eap_psk_isDone;
+	eap->getKey = eap_psk_getKey;
+	eap->isSuccess = eap_psk_isSuccess;
+
+	ret = eap_server_method_register(eap);
+	if (ret)
+		eap_server_method_free(eap);
+	return ret;
+}
