@@ -1,6 +1,6 @@
 /*
- * hostapd / EAP-SIM (draft-haverinen-pppext-eap-sim-16.txt)
- * Copyright (c) 2005, Jouni Malinen <jkmaline@cc.hut.fi>
+ * hostapd / EAP-SIM (RFC 4186)
+ * Copyright (c) 2005-2006, Jouni Malinen <jkmaline@cc.hut.fi>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 2 as
@@ -161,7 +161,7 @@ static int eap_sim_build_encr(struct eap_sm *sm, struct eap_sim_data *data,
 			   data->next_pseudonym);
 		eap_sim_msg_add(msg, EAP_SIM_AT_NEXT_PSEUDONYM,
 				strlen(data->next_pseudonym),
-				data->next_pseudonym,
+				(u8 *) data->next_pseudonym,
 				strlen(data->next_pseudonym));
 	}
 
@@ -170,7 +170,7 @@ static int eap_sim_build_encr(struct eap_sm *sm, struct eap_sim_data *data,
 			   data->next_reauth_id);
 		eap_sim_msg_add(msg, EAP_SIM_AT_NEXT_REAUTH_ID,
 				strlen(data->next_reauth_id),
-				data->next_reauth_id,
+				(u8 *) data->next_reauth_id,
 				strlen(data->next_reauth_id));
 	}
 
@@ -649,16 +649,27 @@ static Boolean eap_sim_isSuccess(struct eap_sm *sm, void *priv)
 }
 
 
-const struct eap_method eap_method_sim =
+int eap_server_sim_register(void)
 {
-	.method = EAP_TYPE_SIM,
-	.name = "SIM",
-	.init = eap_sim_init,
-	.reset = eap_sim_reset,
-	.buildReq = eap_sim_buildReq,
-	.check = eap_sim_check,
-	.process = eap_sim_process,
-	.isDone = eap_sim_isDone,
-	.getKey = eap_sim_getKey,
-	.isSuccess = eap_sim_isSuccess,
-};
+	struct eap_method *eap;
+	int ret;
+
+	eap = eap_server_method_alloc(EAP_SERVER_METHOD_INTERFACE_VERSION,
+				      EAP_VENDOR_IETF, EAP_TYPE_SIM, "SIM");
+	if (eap == NULL)
+		return -1;
+
+	eap->init = eap_sim_init;
+	eap->reset = eap_sim_reset;
+	eap->buildReq = eap_sim_buildReq;
+	eap->check = eap_sim_check;
+	eap->process = eap_sim_process;
+	eap->isDone = eap_sim_isDone;
+	eap->getKey = eap_sim_getKey;
+	eap->isSuccess = eap_sim_isSuccess;
+
+	ret = eap_server_method_register(eap);
+	if (ret)
+		eap_server_method_free(eap);
+	return ret;
+}
