@@ -231,15 +231,26 @@ static Boolean eap_tlv_isSuccess(struct eap_sm *sm, void *priv)
 }
 
 
-const struct eap_method eap_method_tlv =
+int eap_server_tlv_register(void)
 {
-	.method = EAP_TYPE_TLV,
-	.name = "TLV",
-	.init = eap_tlv_init,
-	.reset = eap_tlv_reset,
-	.buildReq = eap_tlv_buildReq,
-	.check = eap_tlv_check,
-	.process = eap_tlv_process,
-	.isDone = eap_tlv_isDone,
-	.isSuccess = eap_tlv_isSuccess,
-};
+	struct eap_method *eap;
+	int ret;
+
+	eap = eap_server_method_alloc(EAP_SERVER_METHOD_INTERFACE_VERSION,
+				      EAP_VENDOR_IETF, EAP_TYPE_TLV, "TLV");
+	if (eap == NULL)
+		return -1;
+
+	eap->init = eap_tlv_init;
+	eap->reset = eap_tlv_reset;
+	eap->buildReq = eap_tlv_buildReq;
+	eap->check = eap_tlv_check;
+	eap->process = eap_tlv_process;
+	eap->isDone = eap_tlv_isDone;
+	eap->isSuccess = eap_tlv_isSuccess;
+
+	ret = eap_server_method_register(eap);
+	if (ret)
+		eap_server_method_free(eap);
+	return ret;
+}
