@@ -1,6 +1,5 @@
 /*
- * Host AP (software wireless LAN access point) user space daemon for
- * Host AP kernel driver / IEEE 802.1X Authenticator - EAPOL state machine
+ * hostapd / IEEE 802.1X Authenticator - EAPOL state machine
  * Copyright (c) 2002-2005, Jouni Malinen <jkmaline@cc.hut.fi>
  *
  * This program is free software; you can redistribute it and/or modify
@@ -660,7 +659,7 @@ SM_STEP(CTRL_DIR)
 
 
 struct eapol_state_machine *
-eapol_sm_alloc(hostapd *hapd, struct sta_info *sta)
+eapol_sm_alloc(struct hostapd_data *hapd, struct sta_info *sta)
 {
 	struct eapol_state_machine *sm;
 
@@ -1186,6 +1185,7 @@ static int eapol_sm_get_eap_user(void *ctx, const u8 *identity,
 {
 	struct eapol_state_machine *sm = ctx;
 	const struct hostapd_eap_user *eap_user;
+	int i, count;
 
 	eap_user = hostapd_get_eap_user(sm->hapd->conf, identity,
 					identity_len, phase2);
@@ -1194,9 +1194,13 @@ static int eapol_sm_get_eap_user(void *ctx, const u8 *identity,
 
 	memset(user, 0, sizeof(*user));
 	user->phase2 = phase2;
-	memcpy(user->methods, eap_user->methods,
-	       EAP_USER_MAX_METHODS > EAP_MAX_METHODS ?
-	       EAP_USER_MAX_METHODS : EAP_MAX_METHODS);
+	count = EAP_USER_MAX_METHODS;
+	if (count > EAP_MAX_METHODS)
+		count = EAP_MAX_METHODS;
+	for (i = 0; i < count; i++) {
+		user->methods[i].vendor = eap_user->methods[i].vendor;
+		user->methods[i].method = eap_user->methods[i].method;
+	}
 
 	if (eap_user->password) {
 		user->password = malloc(eap_user->password_len);
