@@ -1,7 +1,7 @@
 /*
- * Host AP (software wireless LAN access point) user space daemon for
+ * hostapd / Initialization and configuration
  * Host AP kernel driver
- * Copyright (c) 2002-2005, Jouni Malinen <jkmaline@cc.hut.fi>
+ * Copyright (c) 2002-2006, Jouni Malinen <jkmaline@cc.hut.fi>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 2 as
@@ -34,13 +34,14 @@
 #include "ctrl_iface.h"
 #include "tls.h"
 #include "eap_sim_db.h"
+#include "eap.h"
 #include "version.h"
 #include "hostap_common.h"
 
 
 struct hapd_interfaces {
 	int count;
-	hostapd **hapd;
+	struct hostapd_iface **iface;
 };
 
 unsigned char rfc1042_header[6] = { 0xaa, 0xaa, 0x03, 0x00, 0x00, 0x00 };
@@ -65,8 +66,6 @@ void hostapd_logger(struct hostapd_data *hapd, const u8 *addr,
 	if (!format)
 		return;
 
-	va_start(ap, fmt);
-
 	if (hapd && hapd->conf) {
 		conf_syslog_level = hapd->conf->logger_syslog_level;
 		conf_stdout_level = hapd->conf->logger_stdout_level;
@@ -119,7 +118,9 @@ void hostapd_logger(struct hostapd_data *hapd, const u8 *addr,
 
 	if ((conf_stdout & module) && level >= conf_stdout_level) {
 		wpa_debug_print_timestamp();
+		va_start(ap, fmt);
 		vprintf(format, ap);
+		va_end(ap);
 		printf("\n");
 	}
 
@@ -143,12 +144,12 @@ void hostapd_logger(struct hostapd_data *hapd, const u8 *addr,
 			priority = LOG_INFO;
 			break;
 		}
+		va_start(ap, fmt);
 		vsyslog(priority, format, ap);
+		va_end(ap);
 	}
 
 	free(format);
-
-	va_end(ap);
 }
 
 
@@ -174,7 +175,7 @@ const char * hostapd_ip_txt(const struct hostapd_ip_addr *addr, char *buf,
 }
 
 
-static void hostapd_deauth_all_stas(hostapd *hapd)
+static void hostapd_deauth_all_stas(struct hostapd_data *hapd)
 {
 #if 0
 	u8 addr[ETH_ALEN];
@@ -191,7 +192,8 @@ static void hostapd_deauth_all_stas(hostapd *hapd)
 
 
 /* This function will be called whenever a station associates with the AP */
-void hostapd_new_assoc_sta(hostapd *hapd, struct sta_info *sta, int reassoc)
+void hostapd_new_assoc_sta(struct hostapd_data *hapd, struct sta_info *sta,
+			   int reassoc)
 {
 	if (hapd->tkip_countermeasures) {
 		hostapd_sta_deauth(hapd, sta->addr,
@@ -244,7 +246,7 @@ static void handle_term(int sig, void *eloop_ctx, void *signal_ctx)
 }
 
 
-static void hostapd_wpa_auth_conf(struct hostapd_config *conf,
+static void hostapd_wpa_auth_conf(struct hostapd_bss_config *conf,
 				  struct wpa_auth_config *wconf)
 {
 	wconf->wpa = conf->wpa;
@@ -269,8 +271,8 @@ static void handle_reload(int sig, void *eloop_ctx, void *signal_ctx)
 	printf("Signal %d received - reloading configuration\n", sig);
 
 	for (i = 0; i < hapds->count; i++) {
-		hostapd *hapd = hapds->hapd[i];
-		newconf = hostapd_config_read(hapd->config_fname);
+		struct hostapd_data *hapd = hapds->iface[i]->bss[0];
+		newconf = hostapd_config_read(hapds->iface[i]->config_fname);
 		if (newconf == NULL) {
 			printf("Failed to read new configuration file - "
 			       "continuing with old.\n");
@@ -280,18 +282,20 @@ static void handle_reload(int sig, void *eloop_ctx, void *signal_ctx)
 		 * items (e.g., open/close sockets, remove stations added to
 		 * deny list, etc.) */
 		radius_client_flush(hapd->radius);
-		hostapd_config_free(hapd->conf);
+		hostapd_config_free(hapd->iconf);
 
-		hostapd_wpa_auth_conf(newconf, &wpa_auth_conf);
+		hostapd_wpa_auth_conf(&newconf->bss[0], &wpa_auth_conf);
 		wpa_reconfig(hapd->wpa_auth, &wpa_auth_conf);
 
-		hapd->conf = newconf;
+		hapd->iconf = newconf;
+		hapd->conf = &newconf->bss[0];
+		hapds->iface[i]->conf = newconf;
 	}
 }
 
 
 #ifdef HOSTAPD_DUMP_STATE
-static void hostapd_dump_state(hostapd *hapd)
+static void hostapd_dump_state(struct hostapd_data *hapd)
 {
 	FILE *f;
 	time_t now;
@@ -380,11 +384,12 @@ static void handle_dump_state(int sig, void *eloop_ctx, void *signal_ctx)
 {
 #ifdef HOSTAPD_DUMP_STATE
 	struct hapd_interfaces *hapds = (struct hapd_interfaces *) eloop_ctx;
-	int i;
+	int i, j;
 
 	for (i = 0; i < hapds->count; i++) {
-		hostapd *hapd = hapds->hapd[i];
-		hostapd_dump_state(hapd);
+		struct hostapd_iface *hapd_iface = hapds->iface[i];
+		for (j = 0; j < hapd_iface->num_bss; j++)
+			hostapd_dump_state(hapd_iface->bss[j]);
 	}
 #endif /* HOSTAPD_DUMP_STATE */
 }
@@ -424,14 +429,6 @@ static void hostapd_cleanup(struct hostapd_data *hapd)
 
 	hostapd_wireless_event_deinit(hapd);
 
-	if (hapd->driver)
-		hostapd_driver_deinit(hapd);
-
-	hostapd_config_free(hapd->conf);
-	hapd->conf = NULL;
-
-	free(hapd->config_fname);
-
 #ifdef EAP_TLS_FUNCS
 	if (hapd->ssl_ctx) {
 		tls_deinit(hapd->ssl_ctx);
@@ -446,7 +443,17 @@ static void hostapd_cleanup(struct hostapd_data *hapd)
 }
 
 
-static int hostapd_flush_old_stations(hostapd *hapd)
+static void hostapd_cleanup_iface(struct hostapd_iface *iface)
+{
+	hostapd_config_free(iface->conf);
+
+	free(iface->config_fname);
+	free(iface->bss);
+	free(iface);
+}
+
+
+static int hostapd_flush_old_stations(struct hostapd_data *hapd)
 {
 	int ret = 0;
 
@@ -608,7 +615,8 @@ static int hostapd_wpa_auth_set_key(void *ctx, const char *alg, const u8 *addr,
 				    int idx, u8 *key, size_t key_len)
 {
 	struct hostapd_data *hapd = ctx;
-	return hostapd_set_encryption(hapd, alg, addr, idx, key, key_len);
+	return hostapd_set_encryption(hapd->conf->iface, hapd, alg, addr, idx,
+				      key, key_len, 1);
 }
 
 
@@ -616,7 +624,7 @@ static int hostapd_wpa_auth_get_seqnum(void *ctx, const u8 *addr, int idx,
 				       u8 *seq)
 {
 	struct hostapd_data *hapd = ctx;
-	return hostapd_get_seqnum(hapd, addr, idx, seq);
+	return hostapd_get_seqnum(hapd->conf->iface, hapd, addr, idx, seq);
 }
 
 
@@ -644,10 +652,11 @@ static int hostapd_wpa_auth_for_each_sta(
 }
 
 
-static int hostapd_setup_interface(struct hostapd_data *hapd)
+static int hostapd_setup_interface(struct hostapd_iface *iface)
 {
-	struct hostapd_config *conf = hapd->conf;
-	u8 ssid[HOSTAPD_SSID_LEN + 1];
+	struct hostapd_data *hapd = iface->bss[0];
+	struct hostapd_bss_config *conf = hapd->conf;
+	u8 ssid[HOSTAPD_MAX_SSID_LEN + 1];
 	int ssid_len, set_ssid;
 	int ret = 0;
 
@@ -668,27 +677,28 @@ static int hostapd_setup_interface(struct hostapd_data *hapd)
 		printf("Could not read SSID from system\n");
 		return -1;
 	}
-	if (conf->ssid_set) {
+	if (conf->ssid.ssid_set) {
 		/*
 		 * If SSID is specified in the config file and it differs
 		 * from what is being used then force installation of the
 		 * new SSID.
 		 */
-		set_ssid = (conf->ssid_len != ssid_len ||
-			    memcmp(conf->ssid, ssid, ssid_len) != 0);
+		set_ssid = (conf->ssid.ssid_len != ssid_len ||
+			    memcmp(conf->ssid.ssid, ssid, ssid_len) != 0);
 	} else {
 		/*
 		 * No SSID in the config file; just use the one we got
 		 * from the system.
 		 */
 		set_ssid = 0;
-		conf->ssid_len = ssid_len;
-		memcpy(conf->ssid, ssid, conf->ssid_len);
-		conf->ssid[conf->ssid_len] = '\0';
+		conf->ssid.ssid_len = ssid_len;
+		memcpy(conf->ssid.ssid, ssid, conf->ssid.ssid_len);
+		conf->ssid.ssid[conf->ssid.ssid_len] = '\0';
 	}
 
 	printf("Using interface %s with hwaddr " MACSTR " and ssid '%s'\n",
-	       hapd->conf->iface, MAC2STR(hapd->own_addr), hapd->conf->ssid);
+	       hapd->conf->iface, MAC2STR(hapd->own_addr),
+	       hapd->conf->ssid.ssid);
 
 	if (hostapd_setup_wpa_psk(conf)) {
 		printf("WPA-PSK setup failed.\n");
@@ -697,8 +707,8 @@ static int hostapd_setup_interface(struct hostapd_data *hapd)
 
 	/* Set SSID for the kernel driver (to be used in beacon and probe
 	 * response frames) */
-	if (set_ssid && hostapd_set_ssid(hapd, (u8 *) conf->ssid,
-					 conf->ssid_len)) {
+	if (set_ssid && hostapd_set_ssid(hapd, (u8 *) conf->ssid.ssid,
+					 conf->ssid.ssid_len)) {
 		printf("Could not set SSID for kernel driver\n");
 		return -1;
 	}
@@ -891,7 +901,7 @@ static void show_version(void)
 		"hostapd v" VERSION_STR "\n"
 		"User space daemon for IEEE 802.11 AP management,\n"
 		"IEEE 802.1X/WPA/WPA2/EAP/RADIUS Authenticator\n"
-		"Copyright (c) 2002-2005, Jouni Malinen <jkmaline@cc.hut.fi> "
+		"Copyright (c) 2002-2006, Jouni Malinen <jkmaline@cc.hut.fi> "
 		"and contributors\n");
 }
 
@@ -915,27 +925,21 @@ static void usage(void)
 }
 
 
-static hostapd * hostapd_init(const char *config_file)
+static struct hostapd_data *
+hostapd_alloc_bss_data(struct hostapd_iface *hapd_iface,
+		       struct hostapd_config *conf,
+		       struct hostapd_bss_config *bss)
 {
-	hostapd *hapd;
+	struct hostapd_data *hapd;
 
 	hapd = malloc(sizeof(*hapd));
-	if (hapd == NULL) {
-		printf("Could not allocate memory for hostapd data\n");
-		goto fail;
-	}
+	if (hapd == NULL)
+		return NULL;
 	memset(hapd, 0, sizeof(*hapd));
 
-	hapd->config_fname = strdup(config_file);
-	if (hapd->config_fname == NULL) {
-		printf("Could not allocate memory for config_fname\n");
-		goto fail;
-	}
-
-	hapd->conf = hostapd_config_read(hapd->config_fname);
-	if (hapd->conf == NULL) {
-		goto fail;
-	}
+	hapd->iconf = conf;
+	hapd->conf = bss;
+	hapd->iface = hapd_iface;
 
 	if (hapd->conf->individual_wep_key_len > 0) {
 		/* use key0 in individual key and key1 in broadcast key */
@@ -945,29 +949,25 @@ static hostapd * hostapd_init(const char *config_file)
 #ifdef EAP_TLS_FUNCS
 	if (hapd->conf->eap_server &&
 	    (hapd->conf->ca_cert || hapd->conf->server_cert)) {
+		struct tls_connection_params params;
+
 		hapd->ssl_ctx = tls_init(NULL);
 		if (hapd->ssl_ctx == NULL) {
 			printf("Failed to initialize TLS\n");
 			goto fail;
 		}
-		if (tls_global_ca_cert(hapd->ssl_ctx, hapd->conf->ca_cert)) {
-			printf("Failed to load CA certificate (%s)\n",
-				hapd->conf->ca_cert);
-			goto fail;
-		}
-		if (tls_global_client_cert(hapd->ssl_ctx,
-					   hapd->conf->server_cert)) {
-			printf("Failed to load server certificate (%s)\n",
-				hapd->conf->server_cert);
-			goto fail;
-		}
-		if (tls_global_private_key(hapd->ssl_ctx,
-					   hapd->conf->private_key,
-					   hapd->conf->private_key_passwd)) {
-			printf("Failed to load private key (%s)\n",
-			       hapd->conf->private_key);
+
+		memset(&params, 0, sizeof(params));
+		params.ca_cert = hapd->conf->ca_cert;
+		params.client_cert = hapd->conf->server_cert;
+		params.private_key = hapd->conf->private_key;
+		params.private_key_passwd = hapd->conf->private_key_passwd;
+
+		if (tls_global_set_params(hapd->ssl_ctx, &params)) {
+			printf("Failed to set TLS parameters\n");
 			goto fail;
 		}
+
 		if (tls_global_set_verify(hapd->ssl_ctx,
 					  hapd->conf->check_crl)) {
 			printf("Failed to enable check_crl\n");
@@ -993,23 +993,79 @@ static hostapd * hostapd_init(const char *config_file)
 		hapd->assoc_ap_state = WAIT_BEACON;
 
 	/* FIX: need to fix this const vs. not */
-	hapd->driver = (struct driver_ops *) hapd->conf->driver;
+	hapd->driver = (struct driver_ops *) hapd->iconf->driver;
 
 	return hapd;
 
 fail:
-	if (hapd) {
-		if (hapd->ssl_ctx)
-			tls_deinit(hapd->ssl_ctx);
-		if (hapd->conf)
-			hostapd_config_free(hapd->conf);
-		free(hapd->config_fname);
-		free(hapd);
+	/* TODO: cleanup allocated resources(?) */
+	free(hapd);
+	return NULL;
+}
+
+
+static struct hostapd_iface * hostapd_init(const char *config_file)
+{
+	struct hostapd_iface *hapd_iface = NULL;
+	struct hostapd_config *conf = NULL;
+	struct hostapd_data *hapd;
+	int i;
+
+	hapd_iface = malloc(sizeof(*hapd_iface));
+	if (hapd_iface == NULL)
+		goto fail;
+	memset(hapd_iface, 0, sizeof(*hapd_iface));
+
+	hapd_iface->config_fname = strdup(config_file);
+	if (hapd_iface->config_fname == NULL)
+		goto fail;
+
+	conf = hostapd_config_read(hapd_iface->config_fname);
+	if (conf == NULL)
+		goto fail;
+	hapd_iface->conf = conf;
+
+	hapd_iface->num_bss = conf->num_bss;
+	hapd_iface->bss = malloc(conf->num_bss *
+				 sizeof(struct hostapd_data *));
+	if (hapd_iface->bss == NULL)
+		goto fail;
+	memset(hapd_iface->bss, 0, conf->num_bss *
+	       sizeof(struct hostapd_data *));
+
+	for (i = 0; i < conf->num_bss; i++) {
+		hapd = hapd_iface->bss[i] =
+			hostapd_alloc_bss_data(hapd_iface, conf,
+					       &conf->bss[i]);
+		if (hapd == NULL)
+			goto fail;
+	}
+
+	return hapd_iface;
+
+fail:
+	if (conf)
+		hostapd_config_free(conf);
+	if (hapd_iface) {
+		for (i = 0; i < hapd_iface->num_bss; i++) {
+			hapd = hapd_iface->bss[i];
+			if (hapd->ssl_ctx)
+				tls_deinit(hapd->ssl_ctx);
+		}
+
+		free(hapd_iface->config_fname);
+		free(hapd_iface);
 	}
 	return NULL;
 }
 
 
+/**
+ * register_drivers - Register driver interfaces
+ *
+ * This function is generated by Makefile (into driver_conf.c) to call all
+ * configured driver interfaces to register them to core hostapd.
+ */
 void register_drivers(void);
 
 int main(int argc, char *argv[])
@@ -1028,6 +1084,8 @@ int main(int argc, char *argv[])
 			break;
 		case 'd':
 			debug++;
+			if (wpa_debug_level > 0)
+				wpa_debug_level--;
 			break;
 		case 'B':
 			daemonize++;
@@ -1054,10 +1112,16 @@ int main(int argc, char *argv[])
 
 	register_drivers();		/* NB: generated by Makefile */
 
+	if (eap_server_register_methods()) {
+		wpa_printf(MSG_ERROR, "Failed to register EAP methods");
+		return -1;
+	}
+
 	interfaces.count = argc - optind;
 
-	interfaces.hapd = malloc(interfaces.count * sizeof(hostapd *));
-	if (interfaces.hapd == NULL) {
+	interfaces.iface = malloc(interfaces.count *
+				  sizeof(struct hostapd_iface *));
+	if (interfaces.iface == NULL) {
 		printf("malloc failed\n");
 		exit(1);
 	}
@@ -1074,18 +1138,18 @@ int main(int argc, char *argv[])
 
 	for (i = 0; i < interfaces.count; i++) {
 		printf("Configuration file: %s\n", argv[optind + i]);
-		interfaces.hapd[i] = hostapd_init(argv[optind + i]);
-		if (!interfaces.hapd[i])
+		interfaces.iface[i] = hostapd_init(argv[optind + i]);
+		if (!interfaces.iface[i])
 			goto out;
 		for (j = 0; j < debug; j++) {
-			if (interfaces.hapd[i]->conf->logger_stdout_level > 0)
-				interfaces.hapd[i]->conf->
+			if (interfaces.iface[i]->bss[0]->conf->
+			    logger_stdout_level > 0)
+				interfaces.iface[i]->bss[0]->conf->
 					logger_stdout_level--;
-			interfaces.hapd[i]->conf->debug++;
+			interfaces.iface[i]->bss[0]->conf->debug++;
 		}
-		if (hostapd_setup_interface(interfaces.hapd[i]))
+		if (hostapd_setup_interface(interfaces.iface[i]))
 			goto out;
-		wpa_debug_level -= interfaces.hapd[0]->conf->debug;
 	}
 
 	if (daemonize && daemon(0, 0)) {
@@ -1098,26 +1162,39 @@ int main(int argc, char *argv[])
 	eloop_run();
 
 	for (i = 0; i < interfaces.count; i++) {
-		hostapd_free_stas(interfaces.hapd[i]);
-		hostapd_flush_old_stations(interfaces.hapd[i]);
+		for (j = 0; j < interfaces.iface[i]->num_bss; j++) {
+			struct hostapd_data *hapd =
+				interfaces.iface[i]->bss[j];
+			hostapd_free_stas(hapd);
+			hostapd_flush_old_stations(hapd);
+		}
 	}
 
 	ret = 0;
 
  out:
 	for (i = 0; i < interfaces.count; i++) {
-		if (!interfaces.hapd[i])
+		if (!interfaces.iface[i])
 			continue;
-
-		hostapd_cleanup(interfaces.hapd[i]);
-		free(interfaces.hapd[i]);
+		for (j = 0; j < interfaces.iface[i]->num_bss; j++) {
+			struct hostapd_data *hapd =
+				interfaces.iface[i]->bss[j];
+			hostapd_cleanup(hapd);
+			if (j == interfaces.iface[i]->num_bss - 1 &&
+			    hapd->driver)
+				hostapd_driver_deinit(hapd);
+			free(hapd);
+		}
+		hostapd_cleanup_iface(interfaces.iface[i]);
 	}
-	free(interfaces.hapd);
+	free(interfaces.iface);
 
 	eloop_destroy();
 
 	closelog();
 
+	eap_server_unregister_methods();
+
 	driver_unregister_all();
 
 	return ret;
