@@ -1,3 +1,18 @@
+/*
+ * hostapd / Initialization and configuration
+ * Host AP kernel driver
+ * Copyright (c) 2002-2006, Jouni Malinen <jkmaline@cc.hut.fi>
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License version 2 as
+ * published by the Free Software Foundation.
+ *
+ * Alternatively, this software may be distributed under the terms of BSD
+ * license.
+ *
+ * See README and COPYING for more details.
+ */
+
 #ifndef HOSTAPD_H
 #define HOSTAPD_H
 
@@ -69,21 +84,31 @@ struct ieee80211_hdr {
 
 extern unsigned char rfc1042_header[6];
 
-typedef struct hostapd_data hostapd;
-
 struct hostap_sta_driver_data {
 	unsigned long rx_packets, tx_packets, rx_bytes, tx_bytes;
+	unsigned long current_tx_rate;
+	unsigned long inactive_msec;
+	unsigned long flags;
+	unsigned long num_ps_buf_frames;
+	unsigned long tx_retry_failed;
+	unsigned long tx_retry_count;
+	int last_rssi;
+	int last_ack_rssi;
 };
 
 struct driver_ops;
 struct wpa_ctrl_dst;
 struct radius_server_data;
 
+/**
+ * struct hostapd_data - hostapd per-BSS data structure
+ */
 struct hostapd_data {
-	struct hostapd_config *conf;
-	char *config_fname;
+	struct hostapd_iface *iface;
+	struct hostapd_config *iconf;
+	struct hostapd_bss_config *conf;
 
-	u8 own_addr[6];
+	u8 own_addr[ETH_ALEN];
 
 	int num_sta; /* number of entries in sta_list */
 	struct sta_info *sta_list; /* STA info list head */
@@ -129,7 +154,20 @@ struct hostapd_data {
 	struct radius_server_data *radius_srv;
 };
 
-void hostapd_new_assoc_sta(hostapd *hapd, struct sta_info *sta, int reassoc);
+
+/**
+ * struct hostapd_iface - hostapd per-interface data structure
+ */
+struct hostapd_iface {
+	char *config_fname;
+	struct hostapd_config *conf;
+
+	int num_bss;
+	struct hostapd_data **bss;
+};
+
+void hostapd_new_assoc_sta(struct hostapd_data *hapd, struct sta_info *sta,
+			   int reassoc);
 void hostapd_logger(struct hostapd_data *hapd, const u8 *addr,
 		    unsigned int module, int level, const char *fmt,
 		    ...) PRINTF_FORMAT(5, 6);
