@@ -1,6 +1,5 @@
 /*
- * Host AP (software wireless LAN access point) user space daemon for
- * Host AP kernel driver / IEEE 802.11 Management
+ * hostapd / IEEE 802.11 Management
  * Copyright (c) 2002-2005, Jouni Malinen <jkmaline@cc.hut.fi>
  *
  * This program is free software; you can redistribute it and/or modify
@@ -32,7 +31,7 @@
 #include "hostap_common.h"
 
 
-static u8 * hostapd_eid_supp_rates(hostapd *hapd, u8 *eid)
+static u8 * hostapd_eid_supp_rates(struct hostapd_data *hapd, u8 *eid)
 {
 	u8 *pos = eid;
 
@@ -47,7 +46,7 @@ static u8 * hostapd_eid_supp_rates(hostapd *hapd, u8 *eid)
 }
 
 
-static u16 hostapd_own_capab_info(hostapd *hapd)
+static u16 hostapd_own_capab_info(struct hostapd_data *hapd)
 {
 	int capab = WLAN_CAPABILITY_ESS;
 	if (hapd->conf->wpa ||
@@ -184,7 +183,7 @@ static void ieee802_11_print_ssid(const u8 *ssid, u8 len)
 
 static void ieee802_11_sta_authenticate(void *eloop_ctx, void *timeout_ctx)
 {
-	hostapd *hapd = eloop_ctx;
+	struct hostapd_data *hapd = eloop_ctx;
 	struct ieee80211_mgmt mgmt;
 
 	if (hapd->assoc_ap_state == WAIT_BEACON)
@@ -220,7 +219,7 @@ static void ieee802_11_sta_authenticate(void *eloop_ctx, void *timeout_ctx)
 
 static void ieee802_11_sta_associate(void *eloop_ctx, void *timeout_ctx)
 {
-	hostapd *hapd = eloop_ctx;
+	struct hostapd_data *hapd = eloop_ctx;
 	u8 buf[256];
 	struct ieee80211_mgmt *mgmt = (struct ieee80211_mgmt *) buf;
 	u8 *p;
@@ -263,7 +262,7 @@ static void ieee802_11_sta_associate(void *eloop_ctx, void *timeout_ctx)
 }
 
 
-static u16 auth_shared_key(hostapd *hapd, struct sta_info *sta,
+static u16 auth_shared_key(struct hostapd_data *hapd, struct sta_info *sta,
 			   u16 auth_transaction, u8 *challenge, int iswep)
 {
 	hostapd_logger(hapd, sta->addr, HOSTAPD_MODULE_IEEE80211,
@@ -322,7 +321,8 @@ static u16 auth_shared_key(hostapd *hapd, struct sta_info *sta,
 }
 
 
-static void send_auth_reply(hostapd *hapd, struct ieee80211_mgmt *mgmt,
+static void send_auth_reply(struct hostapd_data *hapd,
+			    struct ieee80211_mgmt *mgmt,
 			    u16 auth_alg, u16 auth_transaction, u16 resp,
 			    u8 *challenge)
 {
@@ -365,7 +365,8 @@ static void send_auth_reply(hostapd *hapd, struct ieee80211_mgmt *mgmt,
 }
 
 
-static void handle_auth(hostapd *hapd, struct ieee80211_mgmt *mgmt, size_t len)
+static void handle_auth(struct hostapd_data *hapd, struct ieee80211_mgmt *mgmt,
+			size_t len)
 {
 	u16 auth_alg, auth_transaction, status_code;
 	u16 resp = WLAN_STATUS_SUCCESS;
@@ -374,6 +375,7 @@ static void handle_auth(hostapd *hapd, struct ieee80211_mgmt *mgmt, size_t len)
 	u16 fc;
 	u8 *challenge = NULL;
 	u32 session_timeout, acct_interim_interval;
+	int vlan_id = 0;
 
 	if (len < IEEE80211_HDRLEN + sizeof(mgmt->u.auth)) {
 		printf("handle_auth - too short payload (len=%lu)\n",
@@ -447,7 +449,7 @@ static void handle_auth(hostapd *hapd, struct ieee80211_mgmt *mgmt, size_t len)
 
 	res = hostapd_allowed_address(hapd, mgmt->sa, (u8 *) mgmt, len,
 				      &session_timeout,
-				      &acct_interim_interval);
+				      &acct_interim_interval, &vlan_id);
 	if (res == HOSTAPD_ACL_REJECT) {
 		printf("Station " MACSTR " not allowed to authenticate.\n",
 		       MAC2STR(mgmt->sa));
@@ -469,6 +471,13 @@ static void handle_auth(hostapd *hapd, struct ieee80211_mgmt *mgmt, size_t len)
 		resp = WLAN_STATUS_UNSPECIFIED_FAILURE;
 		goto fail;
 	}
+
+	if (vlan_id > 0) {
+		sta->vlan_id = vlan_id;
+		hostapd_logger(hapd, sta->addr, HOSTAPD_MODULE_RADIUS,
+			       HOSTAPD_LEVEL_INFO, "VLAN ID %d", sta->vlan_id);
+	}
+
 	sta->flags &= ~WLAN_STA_PREAUTH;
 	ieee802_1x_notify_pre_auth(sta->eapol_sm, 0);
 
@@ -505,8 +514,8 @@ static void handle_auth(hostapd *hapd, struct ieee80211_mgmt *mgmt, size_t len)
 }
 
 
-static void handle_assoc(hostapd *hapd, struct ieee80211_mgmt *mgmt,
-			 size_t len, int reassoc)
+static void handle_assoc(struct hostapd_data *hapd,
+			 struct ieee80211_mgmt *mgmt, size_t len, int reassoc)
 {
 	u16 capab_info, listen_interval;
 	u16 resp = WLAN_STATUS_SUCCESS;
@@ -576,8 +585,8 @@ static void handle_assoc(hostapd *hapd, struct ieee80211_mgmt *mgmt,
 		goto fail;
 	}
 
-	if (elems.ssid_len != hapd->conf->ssid_len ||
-	    memcmp(elems.ssid, hapd->conf->ssid, elems.ssid_len) != 0) {
+	if (elems.ssid_len != hapd->conf->ssid.ssid_len ||
+	    memcmp(elems.ssid, hapd->conf->ssid.ssid, elems.ssid_len) != 0) {
 		printf("Station " MACSTR " tried to associate with "
 		       "unknown SSID '", MAC2STR(sta->addr));
 		ieee802_11_print_ssid(elems.ssid, elems.ssid_len);
@@ -733,8 +742,8 @@ static void handle_assoc(hostapd *hapd, struct ieee80211_mgmt *mgmt,
 }
 
 
-static void handle_assoc_resp(hostapd *hapd, struct ieee80211_mgmt *mgmt,
-			      size_t len)
+static void handle_assoc_resp(struct hostapd_data *hapd,
+			      struct ieee80211_mgmt *mgmt, size_t len)
 {
 	u16 status_code, aid;
 
@@ -784,8 +793,8 @@ static void handle_assoc_resp(hostapd *hapd, struct ieee80211_mgmt *mgmt,
 }
 
 
-static void handle_disassoc(hostapd *hapd, struct ieee80211_mgmt *mgmt,
-			    size_t len)
+static void handle_disassoc(struct hostapd_data *hapd,
+			    struct ieee80211_mgmt *mgmt, size_t len)
 {
 	struct sta_info *sta;
 
@@ -842,8 +851,8 @@ static void handle_disassoc(hostapd *hapd, struct ieee80211_mgmt *mgmt,
 }
 
 
-static void handle_deauth(hostapd *hapd, struct ieee80211_mgmt *mgmt,
-			  size_t len)
+static void handle_deauth(struct hostapd_data *hapd,
+			  struct ieee80211_mgmt *mgmt, size_t len)
 {
 	struct sta_info *sta;
 
@@ -887,8 +896,8 @@ static void handle_deauth(hostapd *hapd, struct ieee80211_mgmt *mgmt,
 }
 
 
-static void handle_beacon(hostapd *hapd, struct ieee80211_mgmt *mgmt,
-			  size_t len)
+static void handle_beacon(struct hostapd_data *hapd,
+			  struct ieee80211_mgmt *mgmt, size_t len)
 {
 	struct ieee802_11_elems elems;
 
@@ -991,7 +1000,8 @@ void ieee802_11_mgmt(struct hostapd_data *hapd, u8 *buf, size_t len, u16 stype)
 }
 
 
-static void handle_auth_cb(hostapd *hapd, struct ieee80211_mgmt *mgmt,
+static void handle_auth_cb(struct hostapd_data *hapd,
+			   struct ieee80211_mgmt *mgmt,
 			   size_t len, int ok)
 {
 	u16 auth_alg, auth_transaction, status_code;
@@ -1031,7 +1041,8 @@ static void handle_auth_cb(hostapd *hapd, struct ieee80211_mgmt *mgmt,
 }
 
 
-static void handle_assoc_cb(hostapd *hapd, struct ieee80211_mgmt *mgmt,
+static void handle_assoc_cb(struct hostapd_data *hapd,
+			    struct ieee80211_mgmt *mgmt,
 			    size_t len, int reassoc, int ok)
 {
 	u16 status;
@@ -1082,8 +1093,8 @@ static void handle_assoc_cb(hostapd *hapd, struct ieee80211_mgmt *mgmt,
 		new_assoc = 0;
 	sta->flags |= WLAN_STA_ASSOC;
 
-	if (hostapd_sta_add(hapd, sta->addr, sta->aid, sta->capability,
-			    sta->tx_supp_rates)) {
+	if (hostapd_sta_add(hapd->conf->iface, hapd, sta->addr, sta->aid,
+			    sta->capability, sta->tx_supp_rates)) {
 		printf("Could not add station to kernel driver.\n");
 	}
 
