@@ -1,6 +1,5 @@
 /*
- * Host AP (software wireless LAN access point) user space daemon for
- * Host AP kernel driver / IEEE 802.11 authentication (ACL)
+ * hostapd / IEEE 802.11 authentication (ACL)
  * Copyright (c) 2003-2005, Jouni Malinen <jkmaline@cc.hut.fi>
  *
  * This program is free software; you can redistribute it and/or modify
@@ -33,6 +32,7 @@ struct hostapd_cached_radius_acl {
 	struct hostapd_cached_radius_acl *next;
 	u32 session_timeout;
 	u32 acct_interim_interval;
+	int vlan_id;
 };
 
 
@@ -58,9 +58,9 @@ static void hostapd_acl_cache_free(struct hostapd_cached_radius_acl *acl_cache)
 }
 
 
-static int hostapd_acl_cache_get(struct hostapd_data *hapd, u8 *addr,
+static int hostapd_acl_cache_get(struct hostapd_data *hapd, const u8 *addr,
 				 u32 *session_timeout,
-				 u32 *acct_interim_interval)
+				 u32 *acct_interim_interval, int *vlan_id)
 {
 	struct hostapd_cached_radius_acl *entry;
 	time_t now;
@@ -75,6 +75,8 @@ static int hostapd_acl_cache_get(struct hostapd_data *hapd, u8 *addr,
 			if (entry->accepted == HOSTAPD_ACL_ACCEPT_TIMEOUT)
 				*session_timeout = entry->session_timeout;
 			*acct_interim_interval = entry->acct_interim_interval;
+			if (vlan_id)
+				*vlan_id = entry->vlan_id;
 			return entry->accepted;
 		}
 
@@ -94,7 +96,7 @@ static void hostapd_acl_query_free(struct hostapd_acl_query_data *query)
 }
 
 
-static int hostapd_radius_acl_query(hostapd *hapd, u8 *addr,
+static int hostapd_radius_acl_query(struct hostapd_data *hapd, const u8 *addr,
 				    struct hostapd_acl_query_data *query)
 {
 	struct radius_msg *msg;
@@ -147,7 +149,7 @@ static int hostapd_radius_acl_query(hostapd *hapd, u8 *addr,
 	}
 
 	snprintf(buf, sizeof(buf), RADIUS_802_1X_ADDR_FORMAT ":%s",
-		 MAC2STR(hapd->own_addr), hapd->conf->ssid);
+		 MAC2STR(hapd->own_addr), hapd->conf->ssid.ssid);
 	if (!radius_msg_add_attr(msg, RADIUS_ATTR_CALLED_STATION_ID,
 				 (u8 *) buf, strlen(buf))) {
 		printf("Could not add Called-Station-Id\n");
@@ -185,11 +187,14 @@ static int hostapd_radius_acl_query(hostapd *hapd, u8 *addr,
 }
 
 
-int hostapd_allowed_address(hostapd *hapd, u8 *addr, u8 *msg, size_t len,
-			    u32 *session_timeout, u32 *acct_interim_interval)
+int hostapd_allowed_address(struct hostapd_data *hapd, const u8 *addr,
+			    const u8 *msg, size_t len, u32 *session_timeout,
+			    u32 *acct_interim_interval, int *vlan_id)
 {
 	*session_timeout = 0;
 	*acct_interim_interval = 0;
+	if (vlan_id)
+		*vlan_id = 0;
 
 	if (hostapd_maclist_found(hapd->conf->accept_mac,
 				  hapd->conf->num_accept_mac, addr))
@@ -209,7 +214,8 @@ int hostapd_allowed_address(hostapd *hapd, u8 *addr, u8 *msg, size_t len,
 
 		/* Check whether ACL cache has an entry for this station */
 		int res = hostapd_acl_cache_get(hapd, addr, session_timeout,
-						acct_interim_interval);
+						acct_interim_interval,
+						vlan_id);
 		if (res == HOSTAPD_ACL_ACCEPT ||
 		    res == HOSTAPD_ACL_ACCEPT_TIMEOUT)
 			return res;
@@ -265,7 +271,7 @@ int hostapd_allowed_address(hostapd *hapd, u8 *addr, u8 *msg, size_t len,
 }
 
 
-static void hostapd_acl_expire_cache(hostapd *hapd, time_t now)
+static void hostapd_acl_expire_cache(struct hostapd_data *hapd, time_t now)
 {
 	struct hostapd_cached_radius_acl *prev, *entry, *tmp;
 
@@ -294,7 +300,7 @@ static void hostapd_acl_expire_cache(hostapd *hapd, time_t now)
 }
 
 
-static void hostapd_acl_expire_queries(hostapd *hapd, time_t now)
+static void hostapd_acl_expire_queries(struct hostapd_data *hapd, time_t now)
 {
 	struct hostapd_acl_query_data *prev, *entry, *tmp;
 
@@ -325,7 +331,7 @@ static void hostapd_acl_expire_queries(hostapd *hapd, time_t now)
 
 static void hostapd_acl_expire(void *eloop_ctx, void *timeout_ctx)
 {
-	hostapd *hapd = eloop_ctx;
+	struct hostapd_data *hapd = eloop_ctx;
 	time_t now;
 
 	time(&now);
@@ -401,6 +407,8 @@ hostapd_acl_recv_radius(struct radius_msg *msg, struct radius_msg *req,
 				      MAC2STR(query->addr));
 			cache->acct_interim_interval = 0;
 		}
+
+		cache->vlan_id = radius_msg_get_vlanid(msg);
 	} else
 		cache->accepted = HOSTAPD_ACL_REJECT;
 	cache->next = hapd->acl_cache;
@@ -424,7 +432,7 @@ hostapd_acl_recv_radius(struct radius_msg *msg, struct radius_msg *req,
 }
 
 
-int hostapd_acl_init(hostapd *hapd)
+int hostapd_acl_init(struct hostapd_data *hapd)
 {
 	if (radius_client_register(hapd->radius, RADIUS_AUTH,
 				   hostapd_acl_recv_radius, hapd))
@@ -436,7 +444,7 @@ int hostapd_acl_init(hostapd *hapd)
 }
 
 
-void hostapd_acl_deinit(hostapd *hapd)
+void hostapd_acl_deinit(struct hostapd_data *hapd)
 {
 	struct hostapd_acl_query_data *query, *prev;
 
