@@ -1,6 +1,5 @@
 /*
- * Host AP (software wireless LAN access point) user space daemon for
- * Host AP kernel driver / IEEE 802.1X Authenticator
+ * hostapd / IEEE 802.1X Authenticator
  * Copyright (c) 2002-2005, Jouni Malinen <jkmaline@cc.hut.fi>
  *
  * This program is free software; you can redistribute it and/or modify
@@ -14,7 +13,6 @@
  */
 
 #include "includes.h"
-#include <sys/ioctl.h>
 #include <assert.h>
 
 #include "hostapd.h"
@@ -38,8 +36,8 @@ static void ieee802_1x_new_auth_session(struct hostapd_data *hapd,
 					struct sta_info *sta);
 
 
-static void ieee802_1x_send(hostapd *hapd, struct sta_info *sta, u8 type,
-			    u8 *data, size_t datalen)
+static void ieee802_1x_send(struct hostapd_data *hapd, struct sta_info *sta,
+			    u8 type, u8 *data, size_t datalen)
 {
 	u8 *buf;
 	struct ieee802_1x_hdr *xhdr;
@@ -87,8 +85,8 @@ static void ieee802_1x_send(hostapd *hapd, struct sta_info *sta, u8 type,
 }
 
 
-void ieee802_1x_set_sta_authorized(hostapd *hapd, struct sta_info *sta,
-				   int authorized)
+void ieee802_1x_set_sta_authorized(struct hostapd_data *hapd,
+				   struct sta_info *sta, int authorized)
 {
 	if (sta->flags & WLAN_STA_PREAUTH)
 		return;
@@ -242,7 +240,8 @@ void ieee802_1x_tx_req(struct hostapd_data *hapd, struct sta_info *sta)
 }
 
 
-static void ieee802_1x_tx_key_one(hostapd *hapd, struct sta_info *sta,
+static void ieee802_1x_tx_key_one(struct hostapd_data *hapd,
+				  struct sta_info *sta,
 				  int index, int broadcast,
 				  u8 *key_data, size_t key_len)
 {
@@ -354,9 +353,10 @@ void ieee802_1x_tx_key(struct hostapd_data *hapd, struct sta_info *sta)
 
 		/* TODO: set encryption in TX callback, i.e., only after STA
 		 * has ACKed EAPOL-Key frame */
-		if (hostapd_set_encryption(hapd, "WEP", sta->addr, 0, ikey,
-					   hapd->conf->
-					   individual_wep_key_len)) {
+		if (hostapd_set_encryption(hapd->conf->iface, hapd, "WEP",
+					   sta->addr, 0, ikey,
+					   hapd->conf->individual_wep_key_len,
+					   1)) {
 			printf("Could not set individual WEP encryption.\n");
 		}
 
@@ -365,7 +365,8 @@ void ieee802_1x_tx_key(struct hostapd_data *hapd, struct sta_info *sta)
 }
 
 
-static void ieee802_1x_encapsulate_radius(hostapd *hapd, struct sta_info *sta,
+static void ieee802_1x_encapsulate_radius(struct hostapd_data *hapd,
+					  struct sta_info *sta,
 					  u8 *eap, size_t len)
 {
 	struct radius_msg *msg;
@@ -425,7 +426,7 @@ static void ieee802_1x_encapsulate_radius(hostapd *hapd, struct sta_info *sta,
 	}
 
 	snprintf(buf, sizeof(buf), RADIUS_802_1X_ADDR_FORMAT ":%s",
-		 MAC2STR(hapd->own_addr), hapd->conf->ssid);
+		 MAC2STR(hapd->own_addr), hapd->conf->ssid.ssid);
 	if (!radius_msg_add_attr(msg, RADIUS_ATTR_CALLED_STATION_ID,
 				 (u8 *) buf, strlen(buf))) {
 		printf("Could not add Called-Station-Id\n");
@@ -509,8 +510,9 @@ static char *eap_type_text(u8 type)
 }
 
 
-static void handle_eap_response(hostapd *hapd, struct sta_info *sta,
-				struct eap_hdr *eap, u8 *data, size_t len)
+static void handle_eap_response(struct hostapd_data *hapd,
+				struct sta_info *sta, struct eap_hdr *eap,
+				u8 *data, size_t len)
 {
 	u8 type;
 	struct eapol_state_machine *sm = sta->eapol_sm;
@@ -594,8 +596,8 @@ static void handle_eap_response(hostapd *hapd, struct sta_info *sta,
 
 
 /* Process incoming EAP packet from Supplicant */
-static void handle_eap(hostapd *hapd, struct sta_info *sta, u8 *buf,
-		       size_t len)
+static void handle_eap(struct hostapd_data *hapd, struct sta_info *sta,
+		       u8 *buf, size_t len)
 {
 	struct eap_hdr *eap;
 	u16 eap_len;
@@ -781,7 +783,7 @@ void ieee802_1x_receive(struct hostapd_data *hapd, const u8 *sa, const u8 *buf,
 }
 
 
-void ieee802_1x_new_station(hostapd *hapd, struct sta_info *sta)
+void ieee802_1x_new_station(struct hostapd_data *hapd, struct sta_info *sta)
 {
 	struct rsn_pmksa_cache_entry *pmksa;
 
@@ -891,7 +893,8 @@ void ieee802_1x_free_station(struct sta_info *sta)
 }
 
 
-static void ieee802_1x_decapsulate_radius(hostapd *hapd, struct sta_info *sta)
+static void ieee802_1x_decapsulate_radius(struct hostapd_data *hapd,
+					  struct sta_info *sta)
 {
 	u8 *eap;
 	size_t len;
@@ -970,8 +973,9 @@ static void ieee802_1x_decapsulate_radius(hostapd *hapd, struct sta_info *sta)
 }
 
 
-static void ieee802_1x_get_keys(hostapd *hapd, struct sta_info *sta,
-				struct radius_msg *msg, struct radius_msg *req,
+static void ieee802_1x_get_keys(struct hostapd_data *hapd,
+				struct sta_info *sta, struct radius_msg *msg,
+				struct radius_msg *req,
 				u8 *shared_secret, size_t shared_secret_len)
 {
 	struct radius_ms_mppe_keys *keys;
@@ -1219,6 +1223,25 @@ ieee802_1x_receive_auth(struct radius_msg *msg, struct radius_msg *req,
 
 	switch (msg->hdr->code) {
 	case RADIUS_CODE_ACCESS_ACCEPT:
+		if (sta->ssid->dynamic_vlan == DYNAMIC_VLAN_DISABLED)
+			sta->vlan_id = 0;
+		else
+			sta->vlan_id = radius_msg_get_vlanid(msg);
+		if (sta->vlan_id > 0) {
+			hostapd_logger(hapd, sta->addr,
+				       HOSTAPD_MODULE_RADIUS,
+				       HOSTAPD_LEVEL_INFO,
+				       "VLAN ID %d", sta->vlan_id);
+		} else if (sta->ssid->dynamic_vlan == DYNAMIC_VLAN_REQUIRED) {
+			sta->eapol_sm->authFail = TRUE;
+			hostapd_logger(hapd, sta->addr,
+				       HOSTAPD_MODULE_IEEE8021X,
+				       HOSTAPD_LEVEL_INFO, "authentication "
+				       "server did not include required VLAN "
+				       "ID in Access-Accept");
+			break;
+		}
+
 		/* RFC 3580, Ch. 3.17 */
 		if (session_timeout_set && termination_action ==
 		    RADIUS_TERMINATION_ACTION_RADIUS_REQUEST) {
@@ -1275,7 +1298,8 @@ ieee802_1x_receive_auth(struct radius_msg *msg, struct radius_msg *req,
 
 /* Handler for EAPOL Backend Authentication state machine sendRespToServer.
  * Forward the EAP Response from Supplicant to Authentication Server. */
-void ieee802_1x_send_resp_to_server(hostapd *hapd, struct sta_info *sta)
+void ieee802_1x_send_resp_to_server(struct hostapd_data *hapd,
+				    struct sta_info *sta)
 {
 	struct eapol_state_machine *sm = sta->eapol_sm;
 	if (sm == NULL)
@@ -1350,7 +1374,7 @@ void ieee802_1x_dump_state(FILE *f, const char *prefix, struct sta_info *sta)
 #endif /* HOSTAPD_DUMP_STATE */
 
 
-static int ieee802_1x_rekey_broadcast(hostapd *hapd)
+static int ieee802_1x_rekey_broadcast(struct hostapd_data *hapd)
 {
 	if (hapd->conf->default_wep_key_len < 1)
 		return 0;
@@ -1409,10 +1433,10 @@ static void ieee802_1x_rekey(void *eloop_ctx, void *timeout_ctx)
 
 	/* TODO: Could setup key for RX here, but change default TX keyid only
 	 * after new broadcast key has been sent to all stations. */
-	if (hostapd_set_encryption(hapd, "WEP", NULL,
+	if (hostapd_set_encryption(hapd->conf->iface, hapd, "WEP", NULL,
 				   hapd->default_wep_key_idx,
 				   hapd->default_wep_key,
-				   hapd->conf->default_wep_key_len)) {
+				   hapd->conf->default_wep_key_len, 1)) {
 		hostapd_logger(hapd, NULL, HOSTAPD_MODULE_IEEE8021X,
 			       HOSTAPD_LEVEL_WARNING, "failed to configure a "
 			       "new broadcast key");
@@ -1430,12 +1454,12 @@ static void ieee802_1x_rekey(void *eloop_ctx, void *timeout_ctx)
 }
 
 
-int ieee802_1x_init(hostapd *hapd)
+int ieee802_1x_init(struct hostapd_data *hapd)
 {
 	int i;
 
 	if ((hapd->conf->ieee802_1x || hapd->conf->wpa) &&
-	    hostapd_set_ieee8021x(hapd, 1))
+	    hostapd_set_ieee8021x(hapd->conf->iface, hapd, 1))
 		return -1;
 
 	if (radius_client_register(hapd->radius, RADIUS_AUTH,
@@ -1444,7 +1468,8 @@ int ieee802_1x_init(hostapd *hapd)
 
 	if (hapd->conf->default_wep_key_len) {
 		for (i = 0; i < 4; i++)
-			hostapd_set_encryption(hapd, "none", NULL, i, NULL, 0);
+			hostapd_set_encryption(hapd->conf->iface, hapd,
+					       "none", NULL, i, NULL, 0, 0);
 
 		ieee802_1x_rekey(hapd, NULL);
 
@@ -1456,11 +1481,11 @@ int ieee802_1x_init(hostapd *hapd)
 }
 
 
-void ieee802_1x_deinit(hostapd *hapd)
+void ieee802_1x_deinit(struct hostapd_data *hapd)
 {
 	if (hapd->driver != NULL &&
 	    (hapd->conf->ieee802_1x || hapd->conf->wpa))
-		hostapd_set_ieee8021x(hapd, 0);
+		hostapd_set_ieee8021x(hapd->conf->iface, hapd, 0);
 }
 
 
@@ -1486,8 +1511,8 @@ static void ieee802_1x_new_auth_session(struct hostapd_data *hapd,
 }
 
 
-int ieee802_1x_tx_status(hostapd *hapd, struct sta_info *sta, u8 *buf,
-			 size_t len, int ack)
+int ieee802_1x_tx_status(struct hostapd_data *hapd, struct sta_info *sta,
+			 u8 *buf, size_t len, int ack)
 {
 	struct ieee80211_hdr *hdr;
 	struct ieee802_1x_hdr *xhdr;
