@@ -404,11 +404,12 @@ static int radius_server_request(struct radius_server_data *data,
 				 struct radius_msg *msg,
 				 struct sockaddr *from, socklen_t fromlen,
 				 struct radius_client *client,
-				 const char *from_addr, int from_port)
+				 const char *from_addr, int from_port,
+				 struct radius_session *force_sess)
 {
 	u8 *eap = NULL;
 	size_t eap_len;
-	int res, state_included;
+	int res, state_included = 0;
 	u8 statebuf[4], resp_id;
 	unsigned int state;
 	struct radius_session *sess;
@@ -417,15 +418,19 @@ static int radius_server_request(struct radius_server_data *data,
 
 	/* TODO: Implement duplicate packet processing */
 
-	res = radius_msg_get_attr(msg, RADIUS_ATTR_STATE, statebuf,
-				  sizeof(statebuf));
-	state_included = res >= 0;
-	if (res == sizeof(statebuf)) {
-		state = (statebuf[0] << 24) | (statebuf[1] << 16) |
-			(statebuf[2] << 8) | statebuf[3];
-		sess = radius_server_get_session(client, state);
-	} else {
-		sess = NULL;
+	if (force_sess)
+		sess = force_sess;
+	else {
+		res = radius_msg_get_attr(msg, RADIUS_ATTR_STATE, statebuf,
+					  sizeof(statebuf));
+		state_included = res >= 0;
+		if (res == sizeof(statebuf)) {
+			state = (statebuf[0] << 24) | (statebuf[1] << 16) |
+				(statebuf[2] << 8) | statebuf[3];
+			sess = radius_server_get_session(client, state);
+		} else {
+			sess = NULL;
+		}
 	}
 
 	if (sess) {
@@ -620,7 +625,8 @@ static void radius_server_receive_auth(int sock, void *eloop_ctx,
 	}
 
 	if (radius_server_request(data, msg, (struct sockaddr *) &from,
-				  fromlen, client, abuf, from_port) == -2)
+				  fromlen, client, abuf, from_port, NULL) ==
+	    -2)
 		return; /* msg was stored with the session */
 
 fail:
@@ -1056,6 +1062,7 @@ static int radius_server_get_eap_user(void *ctx, const u8 *identity,
 {
 	struct radius_session *sess = ctx;
 	const struct hostapd_eap_user *eap_user;
+	int i, count;
 
 	eap_user = hostapd_get_eap_user(sess->server->hostapd_conf, identity,
 					identity_len, phase2);
@@ -1063,9 +1070,13 @@ static int radius_server_get_eap_user(void *ctx, const u8 *identity,
 		return -1;
 
 	memset(user, 0, sizeof(*user));
-	memcpy(user->methods, eap_user->methods,
-	       EAP_USER_MAX_METHODS > EAP_MAX_METHODS ?
-	       EAP_USER_MAX_METHODS : EAP_MAX_METHODS);
+	count = EAP_USER_MAX_METHODS;
+	if (count > EAP_MAX_METHODS)
+		count = EAP_MAX_METHODS;
+	for (i = 0; i < count; i++) {
+		user->methods[i].vendor = eap_user->methods[i].vendor;
+		user->methods[i].method = eap_user->methods[i].method;
+	}
 
 	if (eap_user->password) {
 		user->password = malloc(eap_user->password_len);
@@ -1114,8 +1125,10 @@ void radius_server_eap_pending_cb(struct radius_server_data *data, void *ctx)
 			break;
 	}
 
-	if (sess == NULL)
+	if (sess == NULL) {
+		RADIUS_DEBUG("No session matched callback ctx");
 		return;
+	}
 
 	msg = sess->last_msg;
 	sess->last_msg = NULL;
@@ -1124,7 +1137,7 @@ void radius_server_eap_pending_cb(struct radius_server_data *data, void *ctx)
 				  (struct sockaddr *) &sess->last_from,
 				  sess->last_fromlen, cli,
 				  sess->last_from_addr,
-				  sess->last_from_port) == -2)
+				  sess->last_from_port, sess) == -2)
 		return; /* msg was stored with the session */
 
 	radius_msg_free(msg);
