@@ -2005,15 +2005,11 @@ static void __wpa_send_eapol(struct wpa_authenticator *wpa_auth,
 	else
 		version = WPA_KEY_INFO_TYPE_HMAC_MD5_RC4;
 
-	if (sm->wpa == WPA_VERSION_WPA && pairwise) {
-		/* WPA does not include GTK in 4-Way Handshake */
-		gtk = NULL;
-		gtk_len = 0;
-
-		/* key_rsc is for group key, so mask it out in case of
-		 * WPA Pairwise key negotiation. */
-		key_rsc = NULL;
-	}
+	wpa_printf(MSG_DEBUG, "WPA: Send EAPOL(secure=%d mic=%d ack=%d "
+		   "install=%d pairwise=%d ie_len=%d gtk_len=%d keyidx=%d "
+		   "encr=%d)",
+		   secure, mic, ack, install, pairwise, ie_len, gtk_len,
+		   keyidx, encr);
 
 	key_data_len = ie_len + gtk_len;
 
@@ -2053,11 +2049,8 @@ static void __wpa_send_eapol(struct wpa_authenticator *wpa_auth,
 		key_info |= WPA_KEY_INFO_KEY_TYPE;
 	if (encr && sm->wpa == WPA_VERSION_WPA2)
 		key_info |= WPA_KEY_INFO_ENCR_KEY_DATA;
-	if (sm->wpa != WPA_VERSION_WPA2) {
-		if (pairwise)
-			keyidx = 0;
+	if (sm->wpa != WPA_VERSION_WPA2)
 		key_info |= keyidx << WPA_KEY_INFO_KEY_INDEX_SHIFT;
-	}
 	key->key_info = htons(key_info);
 
 	alg = pairwise ? sm->pairwise : wpa_auth->conf.wpa_group;
@@ -2469,10 +2462,11 @@ SM_STATE(WPA_PTK, PTKCALCNEGOTIATING2)
 
 SM_STATE(WPA_PTK, PTKINITNEGOTIATING)
 {
-	u8 rsc[WPA_KEY_RSC_LEN];
+	u8 rsc[WPA_KEY_RSC_LEN], *_rsc, *gtk;
+	size_t gtk_len;
 	struct wpa_authenticator *gsm = sm->wpa_auth;
 	u8 *wpa_ie;
-	int wpa_ie_len;
+	int wpa_ie_len, secure, keyidx;
 
 	SM_ENTRY_MA(WPA_PTK, PTKINITNEGOTIATING, wpa_ptk);
 	sm->TimeoutEvt = FALSE;
@@ -2491,10 +2485,24 @@ SM_STATE(WPA_PTK, PTKINITNEGOTIATING)
 	}
 	wpa_auth_logger(sm->wpa_auth, sm->addr, LOGGER_DEBUG,
 			"sending 3/4 msg of 4-Way Handshake");
-	wpa_send_eapol(sm->wpa_auth, sm, sm->wpa == WPA_VERSION_WPA2 ? 1 : 0,
-		       1, 1, 1, 1, rsc, sm->ANonce,
-		       wpa_ie, wpa_ie_len,
-		       gsm->GTK[gsm->GN - 1], gsm->GTK_len, gsm->GN);
+	if (sm->wpa == WPA_VERSION_WPA2) {
+		/* WPA2 send GTK in the 4-way handshake */
+		secure = 1;
+		gtk = gsm->GTK[gsm->GN - 1];
+		gtk_len = gsm->GTK_len;
+		keyidx = gsm->GN;
+		_rsc = rsc;
+	} else {
+		/* WPA does not include GTK in msg 3/4 */
+		secure = 0;
+		gtk = NULL;
+		gtk_len = 0;
+		keyidx = 0;
+		_rsc = NULL;
+	}
+
+	wpa_send_eapol(sm->wpa_auth, sm, secure, 1, 1, 1, 1, _rsc, sm->ANonce,
+		       wpa_ie, wpa_ie_len, gtk, gtk_len, keyidx);
 	sm->TimeoutCtr++;
 }
 
@@ -2914,8 +2922,8 @@ static int wpa_cipher_bits(int cipher)
 
 int wpa_get_mib(struct wpa_authenticator *wpa_auth, char *buf, size_t buflen)
 {
-	int len = 0, i;
-	char pmkid_txt[PMKID_LEN * 2 + 1], *pos, *end;
+	int len = 0;
+	char pmkid_txt[PMKID_LEN * 2 + 1];
 
 	if (wpa_auth == NULL)
 		return len;
@@ -2933,12 +2941,8 @@ int wpa_get_mib(struct wpa_authenticator *wpa_auth, char *buf, size_t buflen)
 				     HOSTAPD_WPA_VERSION_WPA2),
 			wpa_bool_txt(wpa_auth->conf.rsn_preauth));
 
-	pos = pmkid_txt;
-	end = pos + sizeof(pmkid_txt);
-	for (i = 0; i < PMKID_LEN; i++) {
-		pos += snprintf(pos, end - pos, "%02x",
-				wpa_auth->dot11RSNAPMKIDUsed[i]);
-	}
+	wpa_snprintf_hex(pmkid_txt, sizeof(pmkid_txt),
+			 wpa_auth->dot11RSNAPMKIDUsed, PMKID_LEN);
 
 	len += snprintf(buf + len, buflen - len,
 			"dot11RSNAConfigVersion=%u\n"
